/*jshint browser: true */
/*global XLSX, chrome, JSZip */
// 명도대비 기능 ERROR 수정 및 화면 캡쳐기능을 위한 메세지 패싱, 오픈소스 라이선스 컨텍스트 메뉴 추가


(function () {
  "use strict";

  var ctx = document.createElement('canvas'),
    img = new Image();
  ctx.width = '101';
  ctx.height = '101';
  ctx = ctx.getContext('2d');
  var cPRange = 1;

  chrome.browserAction.onClicked.addListener(function () {
    console.log("in bg.js") ;
    chrome.tabs.getSelected(null, function (tab) {
      if (tab.url.indexOf('http') !== 0 && tab.url.indexOf('https')) {
        window.alert('You can check following schemas only: "http://", "https://"');
        return;
      }

      var isIncludeFrame = window.localStorage.isIncludeFrame !== 'false';
      chrome.tabs.sendMessage(tab.id, {
        action: 'execute_' + (isIncludeFrame ? 'with_frames' : 'without_frames')
      }, function (enabled) {});
    });
  });

  function pickColor2(request) {
    var pixels, x = request.x,
      y = request.y,
      xS1, yS1, xS2, yS2, r = (cPRange - 1) / 2,
      pixel = [0, 0, 0],
      n, m;

    xS1 = x - (x - r < 0 ? x : r);
    yS1 = y - (y - r < 0 ? y : r);
    xS2 = cPRange - (x <= r ? r - x : 0) - (x + r > request.win[0] ? r - request.win[0] + x + 1 : 0);
    yS2 = cPRange - (y <= r ? r - y : 0) - (y + r > request.win[1] ? r - request.win[1] + y + 1 : 0);

    ctx.drawImage(img, xS1, yS1, xS2, yS2, 0, 0, xS2, yS2);
    pixels = ctx.getImageData(0, 0, xS2, yS2).data;
    for (n = 0, m = pixels.length; n < m; n += 4) {
      pixel[0] += pixels[n];
      pixel[1] += pixels[n + 1];
      pixel[2] += pixels[n + 2];
    }
    pixel[0] = Math.round(pixel[0] / m * 4);
    pixel[1] = Math.round(pixel[1] / m * 4);
    pixel[2] = Math.round(pixel[2] / m * 4);
    chrome.tabs.executeScript(null, {
      code: request.next + '(\'rgb(' + pixel[0] + ',' + pixel[1] + ',' + pixel[2] + ')\')'
    });
    if (typeof request.callback === 'function') {
      request.callback(pixel);
    }
  }

  function pickColor(request) {
    if (request.capture) {
      chrome.tabs.captureVisibleTab(null, {
        format: 'png'
      }, function (dataUrl) {
        img.onload = function () {
          pickColor2(request);
        };
        img.src = dataUrl;
        dataUrl = null;
      });
    } else {
      img.onload = function () {
        pickColor2(request);
      };
      img.src = request.dataUrl;
    }
  }

  chrome.extension.onMessage.addListener(function (request, sender, sendResponse) {
    if (request.x) {
      pickColor(request);
    }
  });

  chrome.runtime.onMessage.addListener(function (request, sender, sendResponse) {
    if (request.capture == "yes") {
      chrome.tabs.captureVisibleTab(
        null, {
          format: "png",
          quality: 100
        },
        function (dataUrl) {
          sendResponse({
            src: dataUrl
          });
        }
      );
      return true;
    } else if (request.ignoreCommon) {
      if (!!request.ignoreCommon) {
        chrome.tabs.query({
          active: true,
          currentWindow: true
        }, function (tabs) {
          var isIncludeFrame = true;
          chrome.tabs.sendMessage(
            tabs[0].id, {
              action: 'execute_' + (isIncludeFrame ? 'with_frames' : 'without_frames'),
              allowLogging: false,
              ignoreCommon: request.ignoreCommon
            },
            function (res) {
              if (res && res.err && res.message) {
                document.body.innerHTML = res.message;
                document.body.className = 'result';
              } else {
                window.close();
              }
              return;
            }
          );
        })
      } 
    }else if(!!request.zipDown){
      chrome.downloads.download({url: request.url, filename: "K-WAX-Statistics.zip"});
      return;
    }
  });

  chrome.runtime.onInstalled.addListener(function () {
    chrome.contextMenus.create({
      id: "aaaa",
      type: "normal",
      title: "Open Source License",
      contexts: ["browser_action"]
    });
  });

  chrome.contextMenus.onClicked.addListener(function (info, tab) {
    window.open(chrome.extension.getURL('license.html'));
  });





  
/**
 * The reader namespace
 */
var reader = {};
/**
 * Reader app namespace.
 * @type Object
 */
reader.app = {};

reader.app.active = null;

reader.app.init = function () {
  // reader.app.setBrowserIcon(); 0925ykk 시작시 아이콘 변경 모듈 삭제
 // chrome.browserAction.setTitle({
   //   title: 'Toggle On/Off'
//  });
console.log('comein init()')
  reader.app.listen();
};


reader.app.isActive = function () {
  return new Promise(function (resolve, reject) {
    if(reader.app.active !== null){
      resolve(reader.app.active);
      return;
    }
    chrome.storage.sync.get({active: true}, function (data) {
      reader.app.active = data.active;
      console.log('data.active='  + data.active )
      resolve(data.active);
    });
  });
};

/**
 * This will hold restored settings
 * until event page unloads.
 */
//reader.app.restoredSpeachOptions = null;
reader.app.getSpeakOptions = function(){
  return new Promise(function (resolve, reject) {
    //if(reader.app.restoredSpeachOptions !== null){
    //  resolve(reader.app.restoredSpeachOptions);
    //} else {
      var defaults = {
        'enqueue': true,
        'lang': 'ko-KR',
      //  'gender': 'female',   chrome.tts.speak: Voice gender is deprecated and values will be ignored starting in Chrome 71.
        'rate': 1.3,
        'pitch': 1.0
      };
      chrome.storage.sync.get(defaults, function (data) {
        //console.log(data);
        //window.reader.app.restoredSpeachOptions = data;
        try{
          data.rate = parseFloat(data.rate);
        } catch(e){
          data.rate = 1;
        }
        try{
          data.pitch = parseFloat(data.pitch);
        } catch(e){
          data.pitch = 1;
        }
        resolve(data);
      });
    //}
  });
}

//확장프로그램 아이콘을 활성화/비활성화 시키는 처리

reader.app.setBrowserIcon = function () {
  reader.app.isActive().then(function (active) {
    var icon = chrome.extension.getURL('img/icon32.png');
    if (!active) {
      icon = chrome.extension.getURL('img/icon32_off.png');
    }
    chrome.browserAction.setIcon({
      path: icon
    });
  });
};

/*
reader.app.toggleActive = function () {
  console.log('come in toggleActive');
  reader.app.isActive().then(function (active) {
    active = !active;
console.log('active : ' + active)  ;
    reader.app.active = active;
    if(!active){
      reader.app.stopSpeaking();
    }

    chrome.storage.sync.set({active: active}
      //, function (data) {  //active 변수에 active를 실행하고 setBrowserIcon을 수행하라
//      reader.app.setBrowserIcon();
    //}
    );
  });
};
*/


reader.app.listen = function(){
  chrome.runtime.onMessage.addListener(function(request, sender, sendResponse) {

    if(request.payload){
      switch(request.payload){
        case 'isactive':
          reader.app.isActive().then(function (active) {
            sendResponse({active: active});
          });
          return true;
        case 'play':
          reader.app.stopSpeaking();
          reader.app.speak(request.message);
          break;
        case 'stop':
          reader.app.stopSpeaking();
          break;
      }
    } else if(request.parse === "image"){
      var img = new Image();
      img.onload = function () {
        var canvas = Pixastic.process(img, "brightness", {brightness: 0, contrast: 1.5});
        var url = canvas.toDataURL();
        sendResponse({img: url});
      };
      img.src = request.img;
      return true;
    }
  });
};

reader.app.speak = function(message){
  reader.app.isActive()
    .then(reader.app._prepareUtteranceOptions)
    .then(function(opts){
      if(!opts) return;
      //console.log('Preparing: ', message);
      message = reader.app.prepareText(message);
      reader.app._playUtterance(opts, message);
    });
};
reader.app.stopSpeaking = function(){
  chrome.tts.stop();
}

reader.app._prepareUtteranceOptions = function(enabled){
  if(!enabled) return null;
  return reader.app.getSpeakOptions();
}

/**
 * Play the utterance.
 * @param {Object} options - an options for tts engine
 * @param {String} utterance - an utterance to speak
 */
reader.app._playUtterance = function(options, utterance){

  var parts = this.sentencesGenerator(utterance);
  for(var part of parts){
    //console.log(part.length, part);
    chrome.tts.speak(part, options, function() {
      if (chrome.runtime.lastError) {
        console.log('Error: ' + chrome.runtime.lastError.message);
      }
    });
  }
}

/**
 * Prepare a text to be read in senteces.
 * Chrome tts engine has 32,768 character limit. After reaching this limit
 * the msg will be splitted.
 * The split point will occure after last "." before character limit.
 */
reader.app.sentencesGenerator = function* (msg){
  var max = 240; //32768;
  while(msg){
    var exact = !(msg.length > max);

    var _len = Math.min(max, msg.length);
    var _part = msg.substr(0, _len);
    msg = msg.substr(_len);

    if(!exact){
      // look for last space and don't break a word.
      //start with looking for a "." (dot).
      var _lastSpacePos = _part.lastIndexOf('.');
      if(_lastSpacePos > 1){
        msg = _part.substr(_lastSpacePos+1) + msg;
        _part = _part.substr(0, _lastSpacePos+1);
      } else {
        // ther's not dot in the line. Look for last space.
        _lastSpacePos = _part.lastIndexOf(' ');
        msg = _part.substr(_lastSpacePos) + msg;
        _part = _part.substr(0, _lastSpacePos);
      }
    }

    if(_part.length === 0){
      yield null;
    }
    yield _part;
  }
}

/**
 * Prepare text to be read.
 * It will replace special characters like "$", "£" or "?" to its equivalent
 * understandable by the tts engine.
 */
reader.app.prepareText = function (txt) {

  var replace = '$1 ' + chrome.i18n.getMessage('CONST_POINT') + ' $2';
  var pattern = new RegExp(/(\d+)\.(\d+)/gi);
  var fullMatch = txt.match(pattern);
  replace = replace.replace(/\$0/, fullMatch);
  txt = txt.replace(pattern, replace);


  //replace sentences end mark to [ENDSENT] tag to restore later
  txt = txt.replace(/\.\s/gim, '[ENDSENT] ');
  txt = txt.replace(/\.$/gim, '[ENDSENT]');

  //replace all special characters
  //txt = txt.replace(/\./gim, ' ' + chrome.i18n.getMessage('CONST_DOT') + ' ');
  //txt = txt.replace(/,/gim, ' ' + chrome.i18n.getMessage('CONST_COMA') + ' ');
  txt = txt.replace(/!/gim, ' ' + chrome.i18n.getMessage('CONST_EXLM') + ' ');
  txt = txt.replace(/\?/gim, ' ' + chrome.i18n.getMessage('CONST_QUESTM') + ' ');
  txt = txt.replace(/"/gim, ' ' + chrome.i18n.getMessage('CONST_QUOTM') + ' ');
  txt = txt.replace(/\$/gim, ' ' + chrome.i18n.getMessage('CONST_DOLLAR') + ' ');
  txt = txt.replace(/£/gim, ' ' + chrome.i18n.getMessage('CONST_POUND') + ' ');
  txt = txt.replace(/@/gim, ' ' + chrome.i18n.getMessage('CONST_AT') + ' ');

  txt = txt.replace(/\s?\[ENDSENT\]/g, '.');

  //add spaces to all shortcuts eg FBI -> F B I
  var r = new RegExp(/([A-Z]{2,})/g);
  var matches = txt.match(r);
  if (matches && matches.length > 0) {
      for (var i = 0; i < matches.length; i++) {
          var replacement = matches[i].replace(/(\w)/g, '$1');
          txt = txt.replace(matches[i], replacement);
      }
  }

  return txt;
};

reader.app.init();

//chrome.browserAction.onClicked.addListener(reader.app.toggleActive);
//chrome.browserAction.onClicked Event 
// Fired when a browser action icon is clicked. 
// Does not fire if the browser action has a popup.
// 원본에서는 발생하나 k-wax는 browser action 이 popup을 가지고 있으므로 발생하지 않음. 
// 미사용 처리함. 


}());