/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is N-WAX(NHN Web Accessibility eXtension).
 *
 * The Initial Developer of the Original Code is
 * Goonoo Kim (NHN).
 * Portions created by the Initial Developer are Copyright (C) 2011
 * the Initial Developer. All Rights Reserved.
 * 
 * ----------------------------------------------------------------------------
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/*jslint browser: true */
/*global chrome, Components, openDialog, XMLHttpRequest, FormData, KWAX */

(function (g, document) {
  "use strict";

  if (!g.achecker) {
    g.achecker = {};
  }
  if (!g.achecker.Wax) {
    g.achecker.Wax = {};
  }
  var addEvent = function (obj, type, fn) {
    if (obj.addEventListener) {
      obj.addEventListener(type, fn, false);
    } else if (obj.attachEvent) {
      obj["e" + type + fn] = fn;
      obj[type + fn] = function () {
        obj["e" + type + fn](g.event);
      };
      obj.attachEvent("on" + type, obj[type + fn]);
    }
  };
  var ListSection = g.achecker.Wax.ListSection;
  var TableSection = g.achecker.Wax.TableSection;
  var ToolSection = g.achecker.Wax.ToolSection;
  var getElsFromChildNodes = function (pEl, tagName) {
    var els = [],
      i = 0;
    if (pEl.length && pEl.push) {
      for (i = 0; i < pEl.length; i++) {
        els = els.concat(getElsFromChildNodes(pEl[i], tagName));
      }
      return els;
    }
    var n = pEl.childNodes;
    if (n && n.length) {
      for (i = 0; i < n.length; i++) {
        if (n[i].tagName && n[i].tagName.toLowerCase() === tagName.toLowerCase()) {
          els.push(n[i]);
        }
      }
      return els;
    }
    return [];
  };
  var toggleFoldedClass = function (el) {
    el.className = el.className === 'folded' ? '' : 'folded';

    // fix bug: IE8 won't reflow when set data-* attribute
    if (document && document.all) {
      document.body.className = document.body.getAttribute('className');
    }
  };

  var getContrastRatio = function (color1, color2) {
    var l1; // higher value
    var l2; // lower value
    var contrast;
    var l1R, l1G, l1B, l2R, l2G, l2B;

    // error check, check if pound sign was put in field value
    if (color2.indexOf('#') === 0) {
      color2 = color2.substr(1, color2.length - 1);
    }
    if (color1.indexOf('#') === 0) {
      color1 = color1.substr(1, color1.length - 1);
    }

    //Linearised R (for example) = (R/FS)^2.2 where FS is full scale value (255
    //for 8 bit color channels). L1 is the higher value (of text or background)
    //alert(parseInt("0x"+color1.substr(0, 2)));
    //Math.pow(n,x);
    l1R = parseInt(color1.substr(0, 2), 16) / 255;
    if (l1R <= 0.03928) {
      l1R = l1R / 12.92;
    } else {
      l1R = Math.pow(((l1R + 0.055) / 1.055), 2.4);
    }
    l1G = parseInt(color1.substr(2, 2), 16) / 255;
    if (l1G <= 0.03928) {
      l1G = l1G / 12.92;
    } else {
      l1G = Math.pow(((l1G + 0.055) / 1.055), 2.4);
    }
    l1B = parseInt(color1.substr(4, 2), 16) / 255;
    if (l1B <= 0.03928) {
      l1B = l1B / 12.92;
    } else {
      l1B = Math.pow(((l1B + 0.055) / 1.055), 2.4);
    }
    l2R = parseInt(color2.substr(0, 2), 16) / 255;
    if (l2R <= 0.03928) {
      l2R = l2R / 12.92;
    } else {
      l2R = Math.pow(((l2R + 0.055) / 1.055), 2.4);
    }
    l2G = parseInt(color2.substr(2, 2), 16) / 255;
    if (l2G <= 0.03928) {
      l2G = l2G / 12.92;
    } else {
      l2G = Math.pow(((l2G + 0.055) / 1.055), 2.4);
    }
    l2B = parseInt(color2.substr(4, 2), 16) / 255;
    if (l2B <= 0.03928) {
      l2B = l2B / 12.92;
    } else {
      l2B = Math.pow(((l2B + 0.055) / 1.055), 2.4);
    }
    //where L is luminosity and is defined as
    l1 = (0.2126 * l1R) + (0.7152 * l1G) + (0.0722 * l1B); //using linearised R, G, and B value
    l2 = (0.2126 * l2R) + (0.7152 * l2G) + (0.0722 * l2B); //using linearised R, G, and B value
    //and L2 is the lower value.
    l1 = l1 + 0.05;
    l2 = l2 + 0.05;
    if (l1 < l2) {
      var temp = l1;
      l1 = l2;
      l2 = temp;
    }
    l1 = l1 / l2;
    l1 = l1.toFixed(1);
    return l1;
  };


  var getLabelText = function (el) {
    var ELEMENT_NODE = 1;
    var txt = '';
    var nodeType = el.nodeType;
    var tagName = el.tagName ? el.tagName.toUpperCase() : '';
    var attrType = el.getAttribute ? el.getAttribute('type') : '';
    var styleDisplay;

    try {
      var computedStyle = el.currentStyle || el.ownerDocument.defaultView.getComputedStyle(el, null);
      styleDisplay = computedStyle.display;
    } catch (e) {
      styleDisplay = "";
    }

    if (nodeType === ELEMENT_NODE &&
      styleDisplay === 'none') {
      txt += '[__[';
    }

    if (nodeType === ELEMENT_NODE &&
      tagName === "IMG") {
      txt += el.getAttribute('alt');
    } else if (nodeType === ELEMENT_NODE &&
      tagName === "AREA") {
      txt += el.getAttribute('alt');
    } else if (nodeType === ELEMENT_NODE &&
      tagName === "INPUT" &&
      attrType === 'image') {
      txt += el.getAttribute('alt');
    } else if (nodeType === ELEMENT_NODE &&
      tagName === "INPUT" &&
      (attrType === 'submit' ||
        attrType === 'reset' ||
        attrType === 'button')) {
      txt += el.value;
    } else if (nodeType === ELEMENT_NODE &&
      tagName === "INPUT") {
      txt += '';
    } else if (nodeType === ELEMENT_NODE &&
      (tagName === "TEXTAREA" || tagName === "SELECT")) {
      txt += '';
    } else if (nodeType !== ELEMENT_NODE ||
      (tagName !== "SCRIPT" && tagName !== "STYLE")) {
      var cNodes = el.childNodes;
      var TEXT_NODE = 3;
      var i, l;
      for (i = 0, l = cNodes.length; i < l; i++) {
        if (cNodes[i].nodeType === TEXT_NODE) {
          txt += cNodes[i].nodeValue;
        } else {
          txt += getLabelText(cNodes[i]);
        }
      }
    }

    if (nodeType === ELEMENT_NODE &&
      styleDisplay === 'none') {
      txt += ']__]';
    }
    txt = txt ? txt.replace(/^\s+/, '').replace(/\s+$/, '') : '';
    return txt;
  };

  var getLabel = function (element) {
    var currentLabel = "";
    var currentLabelElement;
    var doc = element.ownerDocument;
    var labelElement, labelChilds, i, l, _i, _l;

    labelLoop:
      for (i = 0, l = doc.getElementsByTagName("label").length; i < l; i++) {
        labelElement = doc.getElementsByTagName("label")[i];
        labelChilds = labelElement.childNodes;

        if (labelElement.htmlFor && labelElement.htmlFor === element.id) {
          currentLabelElement = labelElement;
          break labelLoop;
        }
        for (_i = 0, _l = labelChilds.length; _i < _l; _i++) {
          if (labelChilds[_i] === element) {
            currentLabelElement = labelElement;
            break labelLoop;
          }
        }
      }

    if (currentLabelElement) {
      currentLabel = currentLabelElement.innerText;
    }
    if (!currentLabel) {
      if (element.title) {
        currentLabel = element.title;
      } else if (element.id) {
        currentLabel = element.id;
      } else if (element.name) {
        currentLabel = element.name;
      } else {
        currentLabel = "";
      }
    }
    return currentLabel;
  };

  var getTextContent = function (el) {
    var ELEMENT_NODE = 1;
    var txt = '';
    var nodeType = el.nodeType;
    var tagName = el.tagName ? el.tagName.toUpperCase() : '';
    var attrType = el.getAttribute ? el.getAttribute('type') : '';
    var styleDisplay;

    try {
      var computedStyle = el.currentStyle || el.ownerDocument.defaultView.getComputedStyle(el, null);
      styleDisplay = computedStyle.display;
    } catch (e) {
      styleDisplay = "";
    }

    if (nodeType === ELEMENT_NODE &&
      styleDisplay === 'none') {
      txt += '[__[';
    }

    if (nodeType === ELEMENT_NODE &&
      tagName === "IMG") {
      txt += el.getAttribute('alt');
    } else if (nodeType === ELEMENT_NODE &&
      tagName === "AREA") {
      txt += el.getAttribute('alt');
    } else if (nodeType === ELEMENT_NODE &&
      tagName === "INPUT" &&
      attrType === 'image') {
      txt += el.getAttribute('alt');
    } else if (nodeType === ELEMENT_NODE &&
      tagName === "INPUT" &&
      (attrType === 'submit' ||
        attrType === 'reset' ||
        attrType === 'button')) {
      txt += el.value;
    } else if (nodeType === ELEMENT_NODE &&
      tagName === "INPUT") {
      txt += getLabel(el);
    } else if (nodeType === ELEMENT_NODE &&
      (tagName === "TEXTAREA" || tagName === "SELECT")) {
      txt += getLabel(el);
    } else if (nodeType !== ELEMENT_NODE ||
      (tagName !== "SCRIPT" && tagName !== "STYLE")) {
      var cNodes = el.childNodes;
      var TEXT_NODE = 3;
      var i, l;
      for (i = 0, l = cNodes.length; i < l; i++) {
        if (cNodes[i].nodeType === TEXT_NODE) {
          txt += cNodes[i].nodeValue;
        } else {
          txt += getTextContent(cNodes[i]);
        }
      }
    }

    if (nodeType === ELEMENT_NODE &&
      styleDisplay === 'none') {
      txt += ']__]';
    }
    txt = txt ? txt.replace(/^\s+/, '').replace(/\s+$/, '') : '';
    return txt;
  };

  var getAbsolutePath = function (src, url, doc) {
    var newpath, orgPath;
    var baseHref = (function () {
      if (!doc.getElementsByTagName("base").length) {
        return null;
      }

      var baseEl = doc.getElementsByTagName("base")[0];
      return baseEl.href || null;
    }());

    if (baseHref) {
      url = baseHref;
    }

    // remove url querystring
    url = url.replace(/\?.*$/, '');

    if (src && src.indexOf('//') === -1) {
      if (src.substr(0, 3) === '../') {
        newpath = url.substr(0, url.lastIndexOf('/') > 10 ? url.lastIndexOf('/') : url.length);
        while (src.substr(0, 3) === '../') {
          newpath = newpath.substr(0, newpath.lastIndexOf('/') > 10 ? newpath.lastIndexOf('/') : newpath.length);
          src = src.substr(3);
        }
        src = newpath + '/' + src;
      } else if (src.substr(0, 1) === '/') {
        orgPath = url.replace(/^((?:https?\:\/\/|file\:\/\/\/)[^\/]+)\/.*$/, '$1');
        src = orgPath + src;
      } else {
        orgPath = url.substr(0, url.lastIndexOf('/'));
        src = orgPath + '/' + src;
      }
    }
    return src;
  };

  g.achecker.Wax.run = function (cwin, rdoc, isIncludeFrame, frameDocs, discardFrameUrls) {


    var isIEQuirksMode = rdoc && rdoc.compatMode === "BackCompat" && rdoc.documentMode;

    const lines = rdoc.documentElement.innerHTML.replace('\t', '').split('\n').map(Function.prototype.call, String.prototype.trim);

    if (!rdoc.querySelectorAll) {
      // IE7 support for querySelectorAll in 274 bytes. Supports multiple / grouped selectors and the attribute selector
      // with a "for" attribute. http://www.codecouch.com/
      (function (d, s) {
        d = rdoc, s = d.createStyleSheet();
        d.querySelectorAll = function (r, c, i, j, a) {
          a = d.all, c = [], r = r.replace(/\[for\b/gi, '[htmlFor').split(',');
          for (i = r.length; i--;) {
            s.addRule(r[i], 'k:v');
            for (j = a.length; j--;) {
              a[j].currentStyle.k && c.push(a[j]);
            }
            s.removeRule(0);
          }
          return c;
        };
      })();
    }

    return {
      header: (function () {

        //대상/비대상 섹션 컨테이너
        var $aside = rdoc.createElement('aside');
        $aside.className = 'waxFrames';

        var $title = rdoc.createElement('h1');
        $title.className = '';
        $title.innerText = g.achecker.i18n.get('TargetPage');
        $title.textContent = g.achecker.i18n.get('TargetPage');

        //2019.07.30 필요없는코드
        addEvent($title, 'click', function (e) {
          toggleFoldedClass(this);
        });


        var $pages = rdoc.createElement('ul'),
          $pageLink;
        var $topPage = rdoc.createElement('li');
        var $topPageLink = rdoc.createElement('a');
        var i, l, $page;

        $topPageLink.setAttribute('href', decodeURI(cwin.location.href));
        $topPageLink.setAttribute('target', '_blank');
        $topPageLink.innerText = decodeURI(cwin.location.href);
        $topPageLink.textContent = decodeURI(cwin.location.href);
        $topPage.appendChild($topPageLink);
        $pages.appendChild($topPage);
        if (isIncludeFrame) {
          for (i = 0, l = frameDocs.length; i < l; i++) {
            $page = rdoc.createElement('li');
            $pageLink = rdoc.createElement('a');
            $pageLink.setAttribute('href', frameDocs[i].src);
            $pageLink.setAttribute('target', '_blank');
            $pageLink.innerText = frameDocs[i].src;
            $pageLink.textContent = frameDocs[i].src;
            $page.appendChild($pageLink);
            $pages.appendChild($page);
          }
        }

        $aside.appendChild($title);
        $aside.appendChild($pages);

        if (discardFrameUrls && discardFrameUrls.length) {
          $title = rdoc.createElement('h1');
          $title.innerText = g.achecker.i18n.get('NoneTargetPage');
          $title.textContent = g.achecker.i18n.get('NoneTargetPage');
          addEvent($title, 'click', function (e) {
            toggleFoldedClass(this);
          });
          $pages = rdoc.createElement('ul');
          for (i = 0, l = discardFrameUrls.length; i < l; i++) {
            $page = rdoc.createElement('li');
            $pageLink = rdoc.createElement('a');
            $pageLink.setAttribute('href', discardFrameUrls[i]);
            $pageLink.setAttribute('target', '_blank');
            $pageLink.innerText = discardFrameUrls[i];
            $pageLink.textContent = discardFrameUrls[i];
            $page.appendChild($pageLink);
            $pages.appendChild($page);
          }

          $aside.appendChild($title);
          $aside.appendChild(KWAX.func.getGuide(rdoc, '', g.achecker.i18n.get('NoneTargetPageDesc')));
          $aside.appendChild($pages);
        }

        return $aside;
      }()),
      sections: {
        altText: new TableSection(
          cwin,
          rdoc,
          '1_1. ' + g.achecker.i18n.get('No1') + ' (img)',
          'input[type=image],img,area',
          [{
              label: g.achecker.i18n.get('Hidden'),
              width: 45
            },
            {
              label: g.achecker.i18n.get('Preview'),
              width: 90
            },
            {
              label: g.achecker.i18n.get('Element'),
              width: 45
            },
            {
              label: 'alt ' + g.achecker.i18n.get('Contents'),
              width: 70
            },
            {
              label: g.achecker.i18n.get('FullHTML'),
              className: 'elementhtml',
              width: 180
            },
          ],
          isIncludeFrame,
          frameDocs,
          g.achecker.i18n.get('NotApplicable'),
          function (doc, url) {
            var tagName = this.tagName.toLowerCase();
            var data = {
              hidden: '',
              preview: '',
              el: '',
              alt: '',
              html: ''
            };
            var self = this;

            var handleImg = function () {
              var hasAlt = isIEQuirksMode && self.outerHTML ? self.outerHTML.toLowerCase().indexOf("alt=") > 0 : self.getAttribute('alt') !== null;
              data.alt = rdoc.createElement('span');
              if (!hasAlt) {
                data.alt.innerText = 'alt ' + g.achecker.i18n.get('Undefined');
                data.alt.textContent = 'alt ' + g.achecker.i18n.get('Undefined');
              } else if (!self.getAttribute('alt')) {
                data.alt.innerText = 'alt=""';
                data.alt.textContent = 'alt=""';
              } else {
                data.alt.innerText = self.getAttribute('alt');
                data.alt.textContent = self.getAttribute('alt');
              }
              data.el = tagName;

              if (self.getAttribute('longdesc')) {
                var $longdesc = rdoc.createElement('a');
                $longdesc.setAttribute('href',
                  getAbsolutePath(self.getAttribute('longdesc'), url, doc));
                $longdesc.setAttribute('target', '_blank');
                $longdesc.innerText = 'longdesc link';
                $longdesc.textContent = 'longdesc link';
                data.alt.innerText += ' ';
                data.alt.textContent += ' ';
                data.alt.appendChild($longdesc);
              }

              var src = self.getAttribute('src');

              var $container = rdoc.createElement('div');
              $container.style.width = "100px";
              $container.style.margin = "0 auto";
              $container.style.overflow = "hidden";

              var $download = KWAX.func.getDownloadImgElement(rdoc, self, src, url, doc, getAbsolutePath);

              var $img = rdoc.createElement('img');
              $img.setAttribute('alt', data.alt.innerText);
              $img.setAttribute('src', getAbsolutePath(src, url, doc));
              $download.appendChild($img);
              $container.appendChild($download);
              data.preview = $container;

              data.html = self.outerHTML;
              return [
                data.hidden,
                data.preview,
                data.el,
                data.alt,
                data.html
              ];
            };

            switch (tagName) {
              case "input":
                if (this.type.toLowerCase() !== 'image') {
                  break;
                }
                return handleImg();

              case "img":
                return handleImg();

              case "area":
                var hasAlt = isIEQuirksMode && this.outerHTML ? this.outerHTML.toLowerCase().indexOf("alt=") > 0 : this.getAttribute('alt') !== null;
                data.alt = rdoc.createElement('span');
                if (!hasAlt) {
                  data.alt.innerText = 'alt ' + g.achecker.i18n.get('Undefined');
                  data.alt.textContent = 'alt ' + g.achecker.i18n.get('Undefined');
                } else if (!this.getAttribute('alt')) {
                  data.alt.innerText = 'alt=""';
                  data.alt.textContent = 'alt=""';
                } else {
                  data.alt.innerText = this.getAttribute('alt');
                  data.alt.textContent = this.getAttribute('alt');
                }
                data.el = tagName;
                data.html = this.outerHTML;

                return [
                  data.hidden,
                  data.preview,
                  data.el,
                  data.alt,
                  data.html
                ];
            }

            return false;
          },
          function () {
            var tagName = this.tagName;
            switch (tagName) {
              case "IMG":
              case "INPUT":
              case "AREA":
                var hasAlt = isIEQuirksMode && this.outerHTML ? this.outerHTML.toLowerCase().indexOf("alt=") > 0 : this.getAttribute('alt') !== null;

                if (!hasAlt) {
                  return 'fail';
                }
                if (!this.getAttribute('alt')) {
                  return 'warning';
                }
                return 'pass';
            }
          }
        ),

        altTextBG: new TableSection(
          cwin,
          rdoc,
          '1_2. ' + g.achecker.i18n.get('No1') + ' (bg)',
          'body *',
          [{
              label: g.achecker.i18n.get('Hidden'),
              width: 45
            },
            {
              label: g.achecker.i18n.get('Preview'),
              width: 90
            },
            {
              label: g.achecker.i18n.get('Contents'),
              width: 150
            },
            {
              label: g.achecker.i18n.get('FullHTML'),
              className: 'elementhtml',
              width: 180
            },
          ],
          isIncludeFrame,
          frameDocs,
          g.achecker.i18n.get('NotApplicable'),
          function (doc, url) {
            var data = {
              hidden: '',
              preview: '',
              content: '',
              html: ''
            };

            var computedStyle = this.currentStyle || cwin.getComputedStyle(this, null);
            var bgImage = computedStyle.backgroundImage;

            if (bgImage !== 'none') {
             
              var $bg = rdoc.createElement('span');
              var w, h;
              url = bgImage.replace(/^url\("?/, '').replace(/"?\)$/, '');

              try {
                $bg.style.backgroundImage = bgImage;
                $bg.style.backgroundPosition = computedStyle.backgroundPosition;
                $bg.style.backgroundRepeat = computedStyle.backgroundRepeat;
                w = parseInt(computedStyle.width, 10);
                h = parseInt(computedStyle.height, 10);
                $bg.style.width = computedStyle.width;
                $bg.style.height = computedStyle.height;
              } catch (e) {}
              $bg.style.maxWidth = '100px';
              $bg.style.maxHeight = '200px';
              if (w > 100 && h > 200) {
                $bg.style.backgroundSize = 'cover';
              }
              $bg.style.display = 'inline-block';
              $bg.style.overflow = 'hidden';

              var $container = rdoc.createElement('div');
              $container.style.width = "100px";
              $container.style.margin = "0 auto";
              $container.style.overflow = "hidden";

              var $download = KWAX.func.getDownloadBGUrl(rdoc, url);
              $download.appendChild($bg);
              $container.appendChild($download);

              data.preview = $container;
              data.content = getTextContent(this);

              if (data.content.length > 100) {
                data.content = data.content.substr(0, 100) + '.....';
              }

              var fullHTML = this.outerHTML;
              data.html = fullHTML
              if (fullHTML.length > 200) {
                data.html = fullHTML.substr(0, 200) + '.....';
              }

              return [
                data.hidden,
                data.preview,
                data.content,
                data.html
              ];
            }

            return false;
          },
          function () {
            var hasText2 = this.innerText;
            var validate = '';
            var brotherEls = this.parentElement.children;
            var hasBrotherElInnerText = false;
            for (let i = 0; i < brotherEls.length; i++) {
              if (brotherEls[i].innerText) {
                hasBrotherElInnerText = true;
              }
            }
            if (!hasText2) {
              if (hasBrotherElInnerText) {
                validate = 'warning';
              } else if (getTextContent(this)) {
                validate = 'warning';
              } else {
                validate = 'warning';
              }
            } else {
              validate = 'pass';
            }
            return validate;
          }
        ),
        //2019.07.30 타이틀 수정 1_3 => 2.1
        altTextEmbed: new TableSection(
          cwin,
          rdoc,
          '2_1.' + g.achecker.i18n.get('No2') + ' (object)',
          'object,embed,video,audio,canvas,svg',
          [{
              label: g.achecker.i18n.get('Hidden'),
              width: 45
            },
            {
              label: g.achecker.i18n.get('Element'),
              width: 50
            },
            {
              label: g.achecker.i18n.get('FullHTML'),
              className: 'elementhtml',
              width: 180
            },
          ],
          isIncludeFrame,
          frameDocs,
          g.achecker.i18n.get('NotApplicable'),
          function (doc, url) {
            var data = {
              hidden: '',
              el: '',
              html: ''
            };

            var tagName = this.tagName;
            switch (tagName) {
              case "OBJECT":
                if (this.parentNode && this.parentNode.tagName &&
                  (this.parentNode.tagName === 'OBJECT' ||
                    this.parentNode.tagName === 'EMBED')) {
                  return false;
                }

                var fullHTML = this.outerHTML;

                data.el = this.tagName;
                data.html = fullHTML;
                if (fullHTML.length > 200) {
                  data.html = fullHTML.substr(0, 200) + '.....';
                }

                return [
                  data.hidden,
                  data.el,
                  data.html
                ];
              case "EMBED":
                if (this.parentNode && this.parentNode.tagName &&
                  (this.parentNode.tagName === 'OBJECT' ||
                    this.parentNode.tagName === 'EMBED')) {
                  return false;
                }

                var fullHTML = this.outerHTML;

                data.el = this.tagName;
                data.html = fullHTML;
                if (fullHTML.length > 200) {
                  data.html = fullHTML.substr(0, 200) + '.....';
                }

                return [
                  data.hidden,
                  data.el,
                  data.html
                ];
            }

            return false;
          },
          function () {
            var hasText = isIEQuirksMode && this.outerHTML ? this.outerHTML.toLowerCase().indexOf("title=") > 0 : this.getAttribute('title') !== null;
            if (!hasText) {
              return 'fail';
            }
            if (String(this.getAttribute('title')).trim() == "") {
              return 'warning';
            }
            return 'pass';
          }
        ),

        grayscale: new ToolSection(
          cwin,
          rdoc,
          'grayscale',
          '3. ' + g.achecker.i18n.get('No3'),
          function (win, rdoc) {
            var $ul = rdoc.createElement('ul');
            $ul.className = 'grayscale';
            var $tool = rdoc.createElement('li');
            var $tool_btn = rdoc.createElement('button');
            $tool_btn.className = 'off';
            $tool_btn.innerText = g.achecker.i18n.get('ToggleGrayscale');
            $tool_btn.textContent = g.achecker.i18n.get('ToggleGrayscale');
            $tool_btn.onclick = function () {
              var is_on = $tool_btn.getAttribute('data-on') === '1';
              var el = cwin.document.getElementsByTagName("html")[0];
              if (is_on) {
                el.style.webkitFilter = '';
                el.style.mozFilter = '';
                el.style.filter = '';
                $tool_btn.setAttribute('data-on', '');
                $tool_btn.className = 'off';
              } else {
                el.style.webkitFilter = 'grayscale(100%)';
                el.style.mozFilter = 'grayscale(100%)';
                // Unsafe load so remove this line
                // el.style.filter = 'url("data:image/svg+xml;utf8,<svg xmlns=\'http://www.w3.org/2000/svg\'><filter id=\'grayscale\'><feColorMatrix type=\'matrix\' values=\'0.3333 0.3333 0.3333 0 0 0.3333 0.3333 0.3333 0 0 0.3333 0.3333 0.3333 0 0 0 0 0 1 0\'/></filter></svg>#grayscale")'; // FF 3.5+
                el.style.filter = 'grayscale(100%)'; // chrome+
                el.style.filter = 'gray'; // IE
                $tool_btn.setAttribute('data-on', '1');
                $tool_btn.className = 'on';
              }
            };

            $tool.appendChild($tool_btn);
            $tool.appendChild(KWAX.button.backgroundGrayBtn(rdoc));
            $ul.appendChild($tool);

            return $ul;
          }
        ),

        contrast: new ToolSection(
          cwin,
          rdoc,
          'contrast',
          '5. ' + g.achecker.i18n.get('No5'),
          function (win, rdoc) {
            if (!g.achecker.colorInspector) {
              var $res = rdoc.createElement('p');
              $res.className = 'comment';
              $res.innerText = 'Not Supported.';
              $res.textContent = 'Not Supported.';
              return $res;
            }

            var $ul = rdoc.createElement('ul');
            $ul.className = 'contrast';
            var $color1 = rdoc.createElement('li');
            var $color2 = rdoc.createElement('li');
            var $result = rdoc.createElement('li');

            $color1.className = 'color1';
            var $color1_label = rdoc.createElement('span');
            $color1_label.className = 'tit'
            $color1_label.innerText = g.achecker.i18n.get('Foreground') + ': ';
            $color1_label.textContent = g.achecker.i18n.get('Foreground') + ': ';
            var $color1_color = rdoc.createElement('span');
            $color1_color.className = 'color';
            $color1_color.style.backgroundColor = '#000';
            var $color1_val = rdoc.createElement('input');
            $color1_val.className = 'val';
            $color1_val.value = '#000000';
            $color1_val.textContent = '#000000';
            $color1_val.addEventListener("focusout", function () {
              let code = this.value;
              if (code.match('#+[0-9A-Fa-f]{6}')) {
                $color1.getElementsByClassName('color')[0].style.backgroundColor = code;
                $color1.getElementsByClassName('val')[0].value = code;
                $color1.getElementsByClassName('val')[0].textContent = code;
                var contrastRatio = getContrastRatio(code, $color2.getElementsByClassName('val')[0].value);
                if (contrastRatio >= 4.5) {
                  $result.className = 'pass';
                } else if (contrastRatio >= 3) {
                  $result.className = 'warning';
                } else {
                  $result.className = 'fail';
                }
                $result.getElementsByClassName('result')[0].innerText = contrastRatio + ':1';
                $result.getElementsByClassName('result')[0].textContent = contrastRatio + ':1';
                $result.getElementsByClassName('resultText')[0].style.color = code;
              }
            });
            var $color1_btn = rdoc.createElement('button');
            $color1_btn.innerText = g.achecker.i18n.get('SelectForegroundColor');
            $color1_btn.textContent = g.achecker.i18n.get('SelectForegroundColor');
            $color1.appendChild($color1_label);
            $color1.appendChild($color1_color);
            $color1.appendChild(rdoc.createTextNode(' '));
            $color1.appendChild($color1_val);
            $color1.appendChild(rdoc.createTextNode(' '));
            $color1.appendChild($color1_btn);
            $color1_btn.onclick = function () {
              g.achecker.showOverlay();
              g.achecker.colorInspector.startInspect(function (color) {
                g.achecker.hideOverlay();
                $color1.getElementsByClassName('color')[0].style.backgroundColor = color;
                $color1.getElementsByClassName('val')[0].value = color;
                $color1.getElementsByClassName('val')[0].textContent = color;
                var contrastRatio = getContrastRatio(color, $color2.getElementsByClassName('val')[0].value);
                if (contrastRatio >= 4.5) {
                  $result.className = 'pass';
                } else if (contrastRatio >= 3) {
                  $result.className = 'warning';
                } else {
                  $result.className = 'fail';
                }
                $result.getElementsByClassName('result')[0].innerText = contrastRatio + ':1';
                $result.getElementsByClassName('result')[0].textContent = contrastRatio + ':1';
                $result.getElementsByClassName('resultText')[0].style.color = color;
              });
            };

            $color2.className = 'color2';
            var $color2_label = rdoc.createElement('span');
            $color2_label.className = 'tit';
            $color2_label.innerText = g.achecker.i18n.get('Background') + ': ';
            $color2_label.textContent = g.achecker.i18n.get('Background') + ': ';
            var $color2_color = rdoc.createElement('span');
            $color2_color.className = 'color';
            $color2_color.style.backgroundColor = '#FFF';
            var $color2_val = rdoc.createElement('input');
            $color2_val.className = 'val';
            $color2_val.value = '#FFFFFF';
            $color2_val.textContent = '#FFFFFF';
            $color2_val.addEventListener("focusout", function () {
              let code = this.value;
              if (code.match('#+[0-9A-Fa-f]{6}')) {
                $color2.getElementsByClassName('color')[0].style.backgroundColor = code;
                $color2.getElementsByClassName('val')[0].value = code;
                $color2.getElementsByClassName('val')[0].textContent = code;
                var contrastRatio = getContrastRatio(code, $color1.getElementsByClassName('val')[0].value);
                if (contrastRatio >= 4.5) {
                  $result.className = 'pass';
                } else if (contrastRatio >= 3) {
                  $result.className = 'warning';
                } else {
                  $result.className = 'fail';
                }
                $result.getElementsByClassName('result')[0].innerText = contrastRatio + ':1';
                $result.getElementsByClassName('result')[0].textContent = contrastRatio + ':1';
                $result.getElementsByClassName('resultText')[0].style.backgroundColor = code;
              }
            });
            var $color2_btn = rdoc.createElement('button');
            $color2_btn.innerText = g.achecker.i18n.get('SelectBackgroundColor');
            $color2_btn.textContent = g.achecker.i18n.get('SelectBackgroundColor');
            $color2.appendChild($color2_label);
            $color2.appendChild($color2_color);
            $color2.appendChild(rdoc.createTextNode(' '));
            $color2.appendChild($color2_val);
            $color2.appendChild(rdoc.createTextNode(' '));
            $color2.appendChild($color2_btn);
            $color2_btn.onclick = function () {
              g.achecker.showOverlay();
              g.achecker.colorInspector.startInspect(function (color) {
                g.achecker.hideOverlay();
                $color2.getElementsByClassName('color')[0].style.backgroundColor = color;
                $color2.getElementsByClassName('val')[0].value = color;
                $color2.getElementsByClassName('val')[0].textContent = color;
                var contrastRatio = getContrastRatio(color, $color1.getElementsByClassName('val')[0].value);
                if (contrastRatio >= 4.5) {
                  $result.className = 'pass';
                } else if (contrastRatio >= 3) {
                  $result.className = 'warning';
                } else {
                  $result.className = 'fail';
                }
                $result.getElementsByClassName('result')[0].innerText = contrastRatio + ':1';
                $result.getElementsByClassName('result')[0].textContent = contrastRatio + ':1';
                $result.getElementsByClassName('resultText')[0].style.backgroundColor = color;
              });
            };

            $result.className = 'pass';
            var $result_label = rdoc.createElement('span');
            $result_label.className = 'tit';
            $result_label.innerText = g.achecker.i18n.get('Result') + ': ';
            $result_label.textContent = g.achecker.i18n.get('Result') + ': ';
            var $result_result = rdoc.createElement('span');
            $result_result.className = 'result';
            $result_result.innerText = '21:1';
            $result_result.textContent = '21:1';
            var $result_resultText = rdoc.createElement('span');
            $result_resultText.className = 'resultText';
            $result_resultText.innerText = g.achecker.i18n.get('Test');
            $result_resultText.textContent = g.achecker.i18n.get('Test');
            $result.appendChild($result_label);
            $result.appendChild($result_result);
            $result.appendChild(rdoc.createTextNode(' '));
            $result.appendChild($result_resultText);
            $ul.appendChild($color1);
            $ul.appendChild($color2);
            $ul.appendChild($result);

            return $ul;
          }
        ),

        guaranteedUsingKeyboard: new TableSection(
          cwin,
          rdoc,
          '8. ' + g.achecker.i18n.get('No8'),
          'div, span, p, dl, dd',
          [{
              label: g.achecker.i18n.get('Hidden'),
              width: 45
            },
            {
              label: g.achecker.i18n.get('Element'),
              width: 55
            },
            {
              label: g.achecker.i18n.get('FullHTML'),
              className: 'elementhtml',
              width: 180
            },
          ],
          isIncludeFrame,
          frameDocs,
          g.achecker.i18n.get('NotApplicable'),
          function (doc, url) {

            var data = {
              hidden: '',
              el: '',
              html: ''
            };

            var isOverFlowScroll = getComputedStyle(this).overflow.includes("auto") || getComputedStyle(this).overflow.includes("scroll");

            data.el = this.tagName.toLowerCase();
            var fullHTML = this.outerHTML;
            data.html = fullHTML;
            if (fullHTML.length > 200) {
              data.html = fullHTML.substr(0, 200) + '.....';
            }

            if (isOverFlowScroll) {
              return [
                data.hidden,
                data.el,
                data.html
              ];
            } else {
              return false;
            }
          },
          function (doc, url) {

            var isOverFlowScroll = getComputedStyle(this).overflow.includes("auto") || getComputedStyle(this).overflow.includes("scroll");
            var isTabIndex = this.tabIndex > -1 ? true : false;
            var hasChildNodeTabIndex = false;
            var children = this.children;

            function searchChildNodesTabIndex(childNode) {
              let children = childNode.children;
              let hasTabIndex = false;
              for (let j = 0; j < children.length; j++) {
                if (children[j]) {
                  hasTabIndex = true;
                }
              }
              return hasTabIndex;
            }
            for (let i = 0; i < children.length; i++) {
              hasChildNodeTabIndex = searchChildNodesTabIndex(children[i]);
              if (hasChildNodeTabIndex) {
                break;
              }
            }

            if (isOverFlowScroll && !isTabIndex && !hasChildNodeTabIndex && !g.achecker.Wax.isElHidden(this)) {
              return 'fail';
            } else if (isOverFlowScroll && !isTabIndex && !hasChildNodeTabIndex && g.achecker.Wax.isElHidden(this)) {
              return 'warning';
            } else {
              if (isOverFlowScroll && !isTabIndex && hasChildNodeTabIndex) {
                return 'warning';
              }
              return 'pass';
            }
          }
        ),

        kbdFocus: new TableSection(
          cwin,
          rdoc,
          '9. ' + g.achecker.i18n.get('No9'),
          'a, input, buttn, [tabindex]',
          [{
              label: g.achecker.i18n.get('Hidden'),
              width: 45
            },
            {
              label: g.achecker.i18n.get('ErrorType'),
              width: 65
            },
            {
              label: g.achecker.i18n.get('Contents'),
              width: 100
            },
            {
              label: g.achecker.i18n.get('FullHTML'),
              className: 'elementhtml',
              width: 180
            },
          ],
          isIncludeFrame,
          frameDocs,
          g.achecker.i18n.get('RequireConfirmation'),
          function (doc, url) {
            // ignore this section
            return false;
            // try {
            //   // var data = {
            //   //   hidden: '',
            //   //   function: '',
            //   //   elLineNum: '',
            //   //   content: '',
            //   //   html:''
            //   // };
            //   var data = {
            //     hidden: '',
            //     function: '',
            //     content: '',
            //     html:''
            //   };

            //   if(this.getAttribute('tabindex') == -1){
            //     return false;
            //   }

            //   var evtWrapper = this.wrappedJSObject || this;
            //   // var evtWrapper = this;
            //   var hasBlurEvent = false;

            //    // var outlineWidth = this.style.getPropertyValue('outline-width');
            //   // var outLine = this.style.getPropertyValue('outline');
            //   // var outLineColor = this.style.getPropertyValue('outline-color');
            //   this.dispatchEvent(new Event('focus'));
            //   // this.dispatchEvent(new KeyboardEvent('keypress', {'key': 'Tab'}))
            //   if(!this.matches(':focus')){
            //     this.focus();
            //   }

            //   // console.log(this.getAttribute('onfocus') + " onfocus : " +evtWrapper.onfocus);
            //   // if(evtWrapper.onfocus !== null && evtWrapper.onfocus !== undefined){
            //   //   if (evtWrapper.onfocus.toString().indexOf('blur()') > -1) {
            //   //     hasBlurEvent = true;
            //   //   }
            //   // }else if(evtWrapper.onclick !== null && evtWrapper.onclick !== undefined){
            //   //   if (evtWrapper.onclick.toString().indexOf('blur()') > -1) {
            //   //     hasBlurEvent = true;
            //   //   }
            //   // }
            //   if(evtWrapper.getAttribute('onfocus') !== null && evtWrapper.getAttribute('onfocus') !== undefined){
            //     if (evtWrapper.getAttribute('onfocus').toString().indexOf('blur()') > -1) {
            //       hasBlurEvent = true;
            //     }
            //   }else if(evtWrapper.getAttribute('onclick') !== null && evtWrapper.getAttribute('onclick') !== undefined){
            //     if (evtWrapper.getAttribute('onclick').toString().indexOf('blur()') > -1) {
            //       hasBlurEvent = true;
            //     }
            //   }

            //   // if(document.activeElement !== this){
            //   //   this.parentElement.style.display = 'block';
            //   //   this.parentElement.parentElement.style.display = 'block';
            //   // }

            //   var outlineWidth = window.getComputedStyle(this).getPropertyValue('outline-width');
            //   var outLine = window.getComputedStyle(this).getPropertyValue('outline');;
            //   var outLineColor = window.getComputedStyle(this).getPropertyValue('outline-color').replace(/\D/g, "") > 200200200 ? true : false;
            //   var zeroOutlineWidth = outlineWidth === '0' 
            //                         || outlineWidth === '0pt' 
            //                         || outlineWidth === '0px' 
            //                         || outLine.indexOf('white') > -1 ? true : false 
            //                         || outLine.indexOf('255, 255, 255') > -1 ? true : false 
            //                         // || outLine.indexOf('0') > -1 ? true : false
            //                         || outLineColor;

            //   var fullHTML = this.outerHTML;
            //   data.content = getTextContent(this);
            //   //content too long, cut 400
            //   if(data.content.length > 400){
            //     data.content = data.content.substr(0, 400) + '.....';
            //   }
            //   data.html = fullHTML
            //   if (fullHTML.length > 200) {
            //     data.html = fullHTML.substr(0, 200) + '.....';
            //   }
            //   // console.log(outlineWidth);
            //   // console.log(outLine);
            //   // console.log(outLineColor);
            //   // console.log(zeroOutlineWidth);

            //   // if(!hasBlurEvent && !zeroOutlineWidth){
            //   //   return false;
            //   // }

            //   if (hasBlurEvent) {
            //     data.function += 'blur()  ';
            //   }
            //   if (zeroOutlineWidth) {
            //     data.function += 'outline:0  ';
            //   }

            //   return [
            //     data.hidden,
            //     data.function,
            //     // data.elLineNum,
            //     data.content,
            //     data.html
            //   ];
            // } catch (e) {
            //   console.log(e);
            //   return false;
            // }
          },
          function (doc, url) {
            return false;
            //   try {

            //     if(this.getAttribute('tabindex') == -1){
            //       return false;
            //     }

            //     var evtWrapper = this.wrappedJSObject || this;
            //     var hasBlurEvent = false;
            //     // if(evtWrapper.onfocus !== null || evtWrapper.onfocus !== undefined){
            //     //   if (evtWrapper.onfocus.toString().indexOf('blur()') > -1) {
            //     //     hasBlurEvent = true;
            //     //   }
            //     // }else if(evtWrapper.onclick !== null || evtWrapper.onclick !== undefined){
            //     //   if (evtWrapper.onclick.toString().indexOf('blur()') > -1) {
            //     //     hasBlurEvent = true;
            //     //   }
            //     // }

            //     // var outlineWidth = this.style.getPropertyValue('outline-width');
            //     // var outLine = this.style.getPropertyValue('outline');
            //     // var outLineColor = this.style.getPropertyValue('outline-color');
            //     this.dispatchEvent(new Event('focus'));
            //     // this.dispatchEvent(new KeyboardEvent('keypress', {'key': 'Tab'}))
            //     if(!this.matches(':focus')){
            //       this.focus();
            //     }

            //     if(evtWrapper.getAttribute('onfocus') !== null && evtWrapper.getAttribute('onfocus') !== undefined){
            //       if (evtWrapper.getAttribute('onfocus').toString().indexOf('blur()') > -1) {
            //         hasBlurEvent = true;
            //       }
            //     }else if(evtWrapper.getAttribute('onclick') !== null && evtWrapper.getAttribute('onclick') !== undefined){
            //       if (evtWrapper.getAttribute('onclick').toString().indexOf('blur()') > -1) {
            //         hasBlurEvent = true;
            //       }
            //     }

            //     var outlineWidth = window.getComputedStyle(this).getPropertyValue('outline-width');
            //     var outLine = window.getComputedStyle(this).getPropertyValue('outline');
            //     var outLineColor = window.getComputedStyle(this).getPropertyValue('outline-color').replace(/\D/g, "") > 200200200 ? true : false;
            //     var zeroOutlineWidth = outlineWidth === '0' 
            //                           || outlineWidth === '0pt' 
            //                           || outlineWidth === '0px' 
            //                           || outLine.indexOf('white') > -1 ? true : false 
            //                           || outLine.indexOf('255, 255, 255') > -1 ? true : false 
            //                           // || outLine.indexOf('0') > -1 ? true : false 
            //                           || outLineColor;

            //     // console.log(this)
            //     // console.log(outlineWidth);
            //     // console.log(outLine);
            //     // console.log(outLineColor);
            //     // console.log(zeroOutlineWidth);

            //     if (hasBlurEvent) {
            //       return 'fail';
            //     }
            //     //blur는 발견시 fail 확정, focus의 경우 보이지 않는 요소에 대해 불확실 할 수 있으므로 현재 문서에서 활성화된 객체와 비교
            //     if(document.activeElement !== this){
            //       return 'warning'; 
            //     }
            //     if (zeroOutlineWidth) {
            //       return 'fail';
            //     }
            //     return 'pass';
            //   } catch (e) {
            //     return false;
            //   }
          }
        ),

        skipNav: new TableSection(
          cwin,
          rdoc,
          '14. ' + g.achecker.i18n.get('No14'),
          'a[href^="#"]',
          [{
              label: g.achecker.i18n.get('Hidden'),
              width: 45
            },
            {
              label: g.achecker.i18n.get('No'),
              width: 65
            },
            {
              label: g.achecker.i18n.get('Contents'),
              width: 150
            },
            {
              label: g.achecker.i18n.get('Connected'),
              width: 25
            },
            // {label: g.achecker.i18n.get('LineNumber'), width: 25},
            {
              label: g.achecker.i18n.get('FullHTML'),
              className: 'elementhtml',
              width: 180
            }
          ],
          isIncludeFrame,
          frameDocs,
          '-',
          function (doc, url) {
            //        console.log('doc=' + doc);
            //   window.alert("ykk=" + this.href);
            // window.alert("ykk=" + this.textContent);
            var data = {
              hidden: '',
              linkIdx: '',
              content: '',
              isconnet: '',
              html: ''
            };

            if (cwin.document !== doc) {
              return false;
            }


            var href = this.getAttribute('href');
            var text = this.textContent;
            //  alert('this.textContent=' + text)
            var isConnectedLink = href === '#' ? false :
              !!doc.getElementById(href.replace('#', '')) ||
              doc.getElementsByName(href.replace('#', '')).length > 0;
            //    alert(isConnectedLink);
            var linkIdx = 0;

            var checkEls = document.querySelectorAll('a[href^="#"]');
            for (var i = 0; i < checkEls.length; i++) {
              if (this === checkEls[i]) {
                linkIdx = i + 1;
              }
            }
            var includedWord = ['주메뉴', '본문'];

            if (text.indexOf(includedWord[0]) > -1) {
              data.linkIdx = linkIdx + g.achecker.i18n.get('ThLink');
              data.content = '(' + href + ') ' + getTextContent(this);
              data.isconnet = (isConnectedLink ? 'O' : 'X');
              data.html = this.outerHTML;
            } else if (text.indexOf(includedWord[1]) > -1) {
              data.linkIdx = linkIdx + g.achecker.i18n.get('ThLink');
              data.content = '(' + href + ') ' + getTextContent(this);
              data.isconnet = (isConnectedLink ? 'O' : 'X');
              data.html = this.outerHTML;
            } else {
              return false;
            }

            return [
              data.hidden,
              data.linkIdx,
              data.content,
              data.isconnet,
              data.html

            ];
          },
          function (doc, url) {
            var href = this.getAttribute('href');
            var isConnectedLink = href === '#' ? false :
              !!doc.getElementById(href.replace('#', '')) ||
              doc.getElementsByName(href.replace('#', '')).length > 0;
            //      alert("isConnectedLink2=" + isConnectedLink2);
            //  ykk 임시로변경     return isConnectedLink ? 'pass' : 'fail';
            return isConnectedLink ? 'pass' : 'fail';
          }
        ),

        pageTitle: new ListSection(
          cwin,
          rdoc,
          '15_1. ' + g.achecker.i18n.get('No15') + '(<title>)',
          'body',
          null,
          isIncludeFrame,
          frameDocs,
          '-',
          function (doc, url) {
            var $div = rdoc.createElement('div');
            var $res = rdoc.createElement('span');
            var $val = rdoc.createElement('strong');
            var val = doc.title || g.achecker.i18n.get('NoPageTitle');

            $res.innerText = url + ': ';
            $res.textContent = url + ': ';
            $val.innerText = val;
            $val.textContent = val;
            $res.appendChild($val);
            $div.appendChild($res);

            return $div;
          },
          function (doc) {
            var title = doc.title || '';
            var dupCharacters = [
              '::', '||', '--', '@@', '##', '$$', '%%', '&&', '**', '((', '))', '++', '==', '~~',
              ';;', '<<', '>>', '[[', ']]', '★★', '☆☆', '◎◎', '●●', '◆◆', '◇◇', '□□', '■■', '△△',
              '▲▲', '▽▽', '▼▼', '◁◁', '◀◀', '▷▷', '▶▶', '♠♠', '♤♤', '♡♡', '♥♥', '♧♧', '♣♣', '⊙⊙',
              '◈◈', '▣▣', '◐◐', '◑◑', '▒▒', '▤▤', '▥▥', '▨▨', '▧▧', '▦▦', '▩▩', '♨♨', '☏☏', '☎☎'
            ];
            var hasTitle = !!title;
            var hasSpecialCharactersDup = false;
            for (var i = 0; i < dupCharacters.length; i++) {
              if (title.indexOf(dupCharacters[i]) > -1) {
                hasSpecialCharactersDup = true;
                break;
              }
            }

            return hasTitle && !hasSpecialCharactersDup ? 'pass' : 'fail';
          }
        ),

        frameTitle: new TableSection(
          cwin,
          rdoc,
          '15_2. ' + g.achecker.i18n.get('No15') + '(iframe)',
          'iframe',
          [{
              label: g.achecker.i18n.get('Hidden'),
              width: 35
            },
            {
              label: g.achecker.i18n.get('Element'),
              width: 30
            },
            {
              label: g.achecker.i18n.get('Title'),
              width: 80
            },
            {
              label: g.achecker.i18n.get('Contents'),
              width: 100
            },
            {
              label: g.achecker.i18n.get('FullHTML'),
              className: 'elementhtml',
              width: 180
            },
          ],
          isIncludeFrame,
          frameDocs,
          g.achecker.i18n.get('NotApplicable'),
          function () {
            var data = {
              hidden: '',
              el: '',
              title: '',
              url: '',
              html: ''
            };
            var src = this.getAttribute('src');
            var title = this.getAttribute('title');
            data.el = 'iframe';
            var fullHTML = this.outerHTML;
            data.html = fullHTML;
            if (fullHTML.length > 200) {
              data.html = fullHTML.substr(0, 200) + '.....';
            }

            if (src) {
              var $a = rdoc.createElement('a');
              $a.setAttribute('href', src);
              $a.setAttribute('target', '_blank');
              $a.innerText = src;
              $a.textContent = src;
              data.url = $a;
            } else {
              data.url = g.achecker.i18n.get('NoSrc');
            }
            data.title = title || g.achecker.i18n.get('NoTitle');
            return [
              data.hidden,
              data.el,
              data.title,
              data.url,
              data.html
            ];
          },
          function () {
            if (String(this.getAttribute('title')).trim() == "") {
              return 'warning';
            }
            var title = this.getAttribute('title');
            return title ? 'pass' : 'fail';
          }
        ),

        blockTitle: new TableSection(
          cwin,
          rdoc,
          '15_3. ' + g.achecker.i18n.get('No15') + '(<h1>~<h6>)',
          'h1,h2,h3,h4,h5,h6',
          [{
              label: g.achecker.i18n.get('Hidden'),
              width: 45
            },
            {
              label: g.achecker.i18n.get('Element'),
              width: 45
            },
            {
              label: g.achecker.i18n.get('Contents'),
              width: 100
            },
            {
              label: g.achecker.i18n.get('FullHTML'),
              className: 'elementhtml',
              width: 180
            },
          ],
          isIncludeFrame,
          frameDocs,
          g.achecker.i18n.get('RequireConfirmation'),
          function () {
            var data = {
              hidden: '',
              el: '',
              content: '',
              html: ''
            };

            data.hidden = '',
              data.el = this.tagName.toLowerCase(),
              data.content = getTextContent(this);
            var fullHTML = this.outerHTML;
            data.html = fullHTML;
            if (fullHTML.length > 200) {
              data.html = fullHTML.substr(0, 200) + '.....';
            }

            return [
              data.hidden,
              data.el,
              data.content,
              data.html
            ];
          },
          'pass'
          // ignore Valid
          // function(){
          //   // var hasTitle = isIEQuirksMode && this.outerHTML ? this.outerHTML.toLowerCase().indexOf("title=") > 0 : this.getAttribute('title') !== null;
          //   var hasTitle = this.outerHTML ? this.outerHTML.toLowerCase().indexOf("title=") > 0 : this.getAttribute('title') !== null;
          //   if (!hasTitle) {
          //     return 'fail';
          //   }
          //   if (!this.getAttribute('title')) {
          //     return 'warning';
          //   }
          //   return 'pass';
          // }
        ),

        linkText: new TableSection(
          cwin,
          rdoc,
          '16. ' + g.achecker.i18n.get('No16'),
          'a,area',
          [{
              label: g.achecker.i18n.get('Hidden'),
              width: 45
            },
            {
              label: g.achecker.i18n.get('Element'),
              width: 45
            },
            {
              label: g.achecker.i18n.get('Contents'),
              width: 150
            },
            {
              label: g.achecker.i18n.get('FullHTML'),
              className: 'elementhtml',
              width: 180
            },
          ],
          isIncludeFrame,
          frameDocs,
          g.achecker.i18n.get('NotApplicable'),
          function () {
            var data = {
              hidden: '',
              el: '',
              content: '',
              html: ''
            };

            var href = this.getAttribute('href');
            var text = getTextContent(this);
            var title = this.getAttribute('title');

            data.el = this.tagName.toLowerCase();
            data.content = (text || '-') + (title ? ' (title: ' + title + ')' : '');
            let fullHTML = this.outerHTML;
            data.html = fullHTML;
            if (fullHTML.length > 200) {
              data.html = fullHTML.substr(0, 200) + '.....';
            }

            return href ? [
              data.hidden,
              data.el,
              data.content,
              data.html
            ] : false;
          },
          function () {
            let text = getTextContent(this);
            let childNodes = this.children;
            let hasImgTag = () => {
              for (let i = 0; i < childNodes.length; i++) {
                if (childNodes[i].tagName === 'IMG') {
                  return 'pass';
                }
              }
              return 'fail';
            }

            return text ? 'pass' : hasImgTag();
          }
        ),

        pageLang: new ListSection(
          cwin,
          rdoc,
          '17. ' + g.achecker.i18n.get('No17'),
          'html',
          null,
          isIncludeFrame,
          frameDocs,
          '-',
          function (doc, url) {
            var $div = rdoc.createElement('div');
            var isXhtml = this.getAttribute('xmlns');
            var val = '';
            var $res = rdoc.createElement('span');

            // ignore if protocol is not http(s)? or .html extension
            if (url.substr(0, 4).toLowerCase() !== 'http' &&
              url.substr(-5).toLowerCase() !== '.html') {
              return false;
            }
            if (isXhtml && this.getAttribute('xml:lang') && this.getAttribute('lang')) {
              val = 'xml:lang=' + this.getAttribute('xml:lang') + ', lang=' + this.getAttribute('lang');
            } else if (isXhtml && this.getAttribute('xml:lang')) {
              val = 'xml:lang=' + this.getAttribute('xml:lang');
            } else if (isXhtml && this.getAttribute('lang')) {
              val = 'xml:lang=' + g.achecker.i18n.get('None') + ', lang=' + this.getAttribute('lang');
            } else if (!isXhtml && this.getAttribute('lang')) {
              val = 'lang=' + this.getAttribute('lang');
            } else {
              val = g.achecker.i18n.get('NoMainLang');
            }
            $res.innerText = url + ': ';
            $res.textContent = url + ': ';

            var $val = rdoc.createElement('strong');
            $val.innerText = val;
            $val.textContent = val;
            $res.appendChild($val);
            $div.appendChild($res);
            return $div;
          },
          function (doc, url) {
            var isXhtml = this.getAttribute('xmlns');

            if (isXhtml && this.getAttribute('xml:lang')) {
              return 'pass';
            }
            if (isXhtml && this.getAttribute('lang')) {
              return 'warning';
            }
            if (!isXhtml && this.getAttribute('lang')) {
              return 'pass';
            }
            return 'fail';
          }
        ),

        unintendedFunction: new TableSection(
          cwin,
          rdoc,
          '18. ' + g.achecker.i18n.get('No18'),
          'a,area,input,button',
          [{
              label: g.achecker.i18n.get('Hidden'),
              width: 65
            },
            {
              label: g.achecker.i18n.get('Element'),
              width: 40
            },
            // {label: g.achecker.i18n.get('LineNumber'), width: 25},
            {
              label: g.achecker.i18n.get('Event'),
              width: 100
            },
            {
              label: g.achecker.i18n.get('Contents'),
              width: 100
            },
            {
              label: g.achecker.i18n.get('TitleAttribute'),
              width: 100
            },
            {
              label: g.achecker.i18n.get('FullHTML'),
              className: 'elementhtml',
              width: 180
            },
          ],
          isIncludeFrame,
          frameDocs,
          g.achecker.i18n.get('RequireConfirmation'),
          function (doc, url) {
            var data = {
              hidden: '',
              el: '',
              event: '',
              content: '',
              title: '',
              html: ''
            };
            var evtWrapper = this.wrappedJSObject || this;
            // var hasChangeEvent;
            var hasWindowOpenEvent;
            var contentForTest;
            var custom_blankCode = false;
            var commonLink = false; // 일반적인 a 요소 <a href='abcd' target="_blank"
            try {
              hasWindowOpenEvent = evtWrapper.onclick ?
                evtWrapper.onclick.toString().indexOf('window.open') > -1 : false;
            } catch (e) {
              hasWindowOpenEvent = null;
            }
            try {
              if (!!evtWrapper.href) {
                let href = evtWrapper.href;
                if (href.indexOf('javascript') > -1) {
                  if (href.lastIndexOf('_blank') > -1 || evtWrapper.href.toString().indexOf('openPopup') > -1 || evtWrapper.href.toString().indexOf('window.open') > -1 || evtWrapper.href.toString().toLowerCase().indexOf('pop') > -1) {
                    if (evtWrapper.href.toString().indexOf('LayerPop') < 0) { // 2019.08.06 예외 처리 추가 
                      if (!this.className.includes('is-depth')) {
                        custom_blankCode = true;
                      }
                    }
                    // let cut1 = href.substring(href.lastIndexOf('_blank'));
                    // let cut2 = cut1.substring(cut1.indexOf('\''));
                    // let cut3 = cut1.substring(cut1.indexOf('\"'));
                    // custom_blankCode = cut2.match(/(\w)/g) === null || cut3.match(/(\w)/g) === null ? true : false;
                  }
                }
              }
            } catch (e) {
              custom_blankCode = false;
              // console.log('custom_blankCode_error');
            }
           try { // a 요소의 일반 링크도 포함한다. 
              if (evtWrapper.tagName.toLowerCase() === 'a' && !!evtWrapper.href && evtWrapper.target === '_blank') {
                commonLink = true
              }
            } catch (e) {
              commonLink = false;
            }

            data.el = this.tagName.toLowerCase();
            let fullHTML = this.outerHTML
            data.html = fullHTML;
            if (fullHTML.length > 200) {
              data.html = fullHTML.substr(0, 200) + '.....';
            }
            data.content = getTextContent(this);
            contentForTest = data.content.replace(/ /g, '').toLowerCase();
            if (this.getAttribute('title')) {
              data.title = this.getAttribute('title');
            } else if (this.getAttribute('target') === '_blank') {
              data.title = 'target="_blank"';
            } else if (contentForTest.indexOf('새창') > -1 ||
              contentForTest.indexOf('팝업') > -1 ||
              contentForTest.toLowerCase().indexOf('newwin') > -1) {} else {
              data.title = ''
            }

            if (hasWindowOpenEvent) {
              data.event = 'window.open';
            } else if (custom_blankCode) {
              data.event = 'href="_blank"';
            } else if (commonLink) {  //2019.08.07 일반적인 a 요소를 검출대상에 포함시키기 위한 코드 
                  data.event = 'target="_blank"';}
            else {
              return false;
            }

            return [
              data.hidden,
              data.el,
              data.event,
              data.content,
              data.title,
              data.html
            ];
          },
          function () {
            var content = getTextContent(this);
            var href = this.href;
            var fileExt = ['.jpg', '.pdf', '.xlsx', '.xls', '.hwp', '.png', '.zip'];
            try {
              if (this.getAttribute('title').toString().indexOf('새창') > -1 || +
                this.getAttribute('title').toString().indexOf('팝업') > -1 || +
                this.getAttribute('title').toString().toLowerCase().indexOf('new window') > -1) {
                return 'pass';
              }
            } catch (e) {}
            if (this.getAttribute('target') === '_blank') {
              return 'pass';
            }
            if (content.indexOf('새창') > -1 ||
              content.indexOf('팝업') > -1 ||
              content.toLowerCase().indexOf('new window') > -1) {
              return 'pass';
            }
            if (String(this.getAttribute('title')).trim() == "") {
              return 'warning';
            }

            for (var i = 0; i < fileExt.length; i++) {
              if (href.substring(href.lastIndexOf('.')).includes(fileExt[i])) {
                return 'warning';
              }
            }
            return 'fail';
          }
        ),

        table: new TableSection(
          cwin,
          rdoc,
          '20. ' + g.achecker.i18n.get('No20'),
          'table',
          [{
              label: g.achecker.i18n.get('Hidden'),
              width: 55
            },
            {
              label: g.achecker.i18n.get('Structure'),
              className: 'tb_str'
//              ,width: 555  // 20191206 kwax레이어크기줄였을때 table의 크기 줄어들지 않는 문제를 개선하기 위해 width가 삭제함
            }
          ],
          isIncludeFrame,
          frameDocs,
          g.achecker.i18n.get('NotApplicable'),
          function () {
            var data = {
              hidden: '',
              structure: ''
            };
            var childNodes = this.childNodes;
            var $caption = null;
            var $resultCaption = rdoc.createElement('p');
            var $captionTitle = rdoc.createElement('span');
            $captionTitle.innerText = 'Caption : ';
            var $summaryTitle = rdoc.createElement('span');
            $summaryTitle.innerText = 'Summary : ';
            var $resultSummary = rdoc.createElement('p');
            var redCircle = '<svg style="width: 20px; height: 17px; overflow: inherit;"><circle cx="15" cy="12" r="6" stroke="red" stroke-width="3" fill="none"></circle></svg>';
            var i, l;

            for (i = 0; i < childNodes.length; i++) {
              if (childNodes[i].tagName &&
                childNodes[i].tagName.toLowerCase() === 'caption') {
                $caption = childNodes[i];
                break;
              }
            }
            var hasCaption = !!$caption;
            var hasSummary = !!this.getAttribute('summary');

            $resultCaption.innerHTML = redCircle + $captionTitle.outerHTML + (hasCaption ? getTextContent($caption) : g.achecker.i18n.get('None'));
            $resultSummary.innerHTML = redCircle + $summaryTitle.outerHTML + (hasSummary ? this.getAttribute('summary') : g.achecker.i18n.get('None'));

            var $thead = getElsFromChildNodes(this, 'thead');
            var $tfoot = getElsFromChildNodes(this, 'tfoot');
            var $tbody = getElsFromChildNodes(this, 'tbody');
            var $theadTh = getElsFromChildNodes(
              getElsFromChildNodes($thead, 'tr'),
              'th'
            );
            var $tfootTh = getElsFromChildNodes(
              getElsFromChildNodes($tfoot, 'tr'),
              'th'
            );
            var $tbodyTh = getElsFromChildNodes(
              getElsFromChildNodes($tbody, 'tr'),
              'th'
            ).concat(
              getElsFromChildNodes(
                getElsFromChildNodes(this, 'tr'),
                'th'
              )
            );
            var hasTh = !!$theadTh.length || !!$tfootTh.length || $tbodyTh.length;
            var $resultTable = rdoc.createElement('table');
            var $resultThead = rdoc.createElement('thead');
            var $resultTbody = rdoc.createElement('tbody');
            var $resultTheadTr = rdoc.createElement('tr');
            var $resultTheadTh1 = rdoc.createElement('th');
            var $resultTheadTh2 = rdoc.createElement('th');
            var $resultTheadTh3 = rdoc.createElement('th');
            $resultTheadTh1.innerText = '요소';
            $resultTheadTh1.textContent = '요소';
            $resultTheadTh2.innerText = '유무';
            $resultTheadTh2.textContent = '유무';
            $resultTheadTh3.innerText = '제목셀 정보 (scope 속성값)';
            $resultTheadTh3.textContent = '제목셀 정보 (scope 속성값)';
            $resultTheadTr.appendChild($resultTheadTh1);
            $resultTheadTr.appendChild($resultTheadTh2);
            $resultTheadTr.appendChild($resultTheadTh3);
            $resultThead.appendChild($resultTheadTr);
            $resultTable.appendChild($resultThead);
            $resultTable.appendChild($resultTbody);

            var $resultTr = rdoc.createElement('tr');
            var $resultTd1 = rdoc.createElement('td');
            var $resultTd2 = rdoc.createElement('td');
            var $resultTd3 = rdoc.createElement('td');
            var $resultTd3Ul;
            var $th;
            var scope;
            var $infoItem;

            if ($thead.length) {
              $resultTd3Ul = rdoc.createElement('ul');
              $resultTd1.innerText = 'thead';
              $resultTd1.textContent = 'thead';
              $resultTd2.innerText = 'O';
              $resultTd2.textContent = 'O';
              for (i = 0, l = $theadTh.length; i < l; i++) {
                $th = $theadTh[i];
                scope = $th.getAttribute('scope');
                $infoItem = rdoc.createElement('li');

                $infoItem.innerText = getTextContent($th) +
                  (!!scope ? '(' + scope + ')' : '(X)');
                $infoItem.textContent = getTextContent($th) +
                  (!!scope ? '(' + scope + ')' : '(X)');
                $resultTd3Ul.appendChild($infoItem);
              }
              $resultTd3.appendChild($resultTd3Ul);
            } else {
              $resultTd1.innerText = 'thead';
              $resultTd1.textContent = 'thead';
              $resultTd2.innerText = 'X';
              $resultTd2.textContent = 'X';
              $resultTd3.innerText = '-';
              $resultTd3.textContent = '-';
            }
            $resultTr.appendChild($resultTd1);
            $resultTr.appendChild($resultTd2);
            $resultTr.appendChild($resultTd3);
            $resultTbody.appendChild($resultTr);

            $resultTr = rdoc.createElement('tr');
            $resultTd1 = rdoc.createElement('td');
            $resultTd2 = rdoc.createElement('td');
            $resultTd3 = rdoc.createElement('td');
            if ($tfoot.length) {
              $resultTd3Ul = rdoc.createElement('ul');
              $resultTd1.innerText = 'tfoot';
              $resultTd1.textContent = 'tfoot';
              $resultTd2.innerText = 'O';
              $resultTd2.textContent = 'O';
              for (i = 0, l = $tfootTh.length; i < l; i++) {
                $th = $tfootTh[i];
                scope = $th.getAttribute('scope');
                $infoItem = rdoc.createElement('li');

                $infoItem.innerText = getTextContent($th) +
                  (!!scope ? '(' + scope + ')' : '(X)');
                $infoItem.textContent = getTextContent($th) +
                  (!!scope ? '(' + scope + ')' : '(X)');
                $resultTd3Ul.appendChild($infoItem);
              }
              $resultTd3.appendChild($resultTd3Ul);
            } else {
              $resultTd1.innerText = 'tfoot';
              $resultTd1.textContent = 'tfoot';
              $resultTd2.innerText = 'X';
              $resultTd2.textContent = 'X';
              $resultTd3.innerText = '-';
              $resultTd3.textContent = '-';
            }
            $resultTr.appendChild($resultTd1);
            $resultTr.appendChild($resultTd2);
            $resultTr.appendChild($resultTd3);
            $resultTbody.appendChild($resultTr);

            $resultTr = rdoc.createElement('tr');
            $resultTd1 = rdoc.createElement('td');
            $resultTd2 = rdoc.createElement('td');
            $resultTd3 = rdoc.createElement('td');
            if ($tbody.length) {
              $resultTd3Ul = rdoc.createElement('ul');
              $resultTd1.innerText = 'tbody';
              $resultTd1.textContent = 'tbody';
              $resultTd2.innerText = 'O';
              $resultTd2.textContent = 'O';
              for (i = 0, l = $tbodyTh.length; i < l; i++) {
                $th = $tbodyTh[i];
                scope = $th.getAttribute('scope');
                $infoItem = rdoc.createElement('li');

                $infoItem.innerText = getTextContent($th) +
                  (!!scope ? '(' + scope + ')' : '(X)');
                $infoItem.textContent = getTextContent($th) +
                  (!!scope ? '(' + scope + ')' : '(X)');
                $resultTd3Ul.appendChild($infoItem);
              }
              $resultTd3.appendChild($resultTd3Ul);
            } else {
              $resultTd1.innerText = 'tbody';
              $resultTd1.textContent = 'tbody';
              $resultTd2.innerText = 'X';
              $resultTd2.textContent = 'X';
              $resultTd3.innerText = '-';
              $resultTd3.textContent = '-';
            }
            $resultTr.appendChild($resultTd1);
            $resultTr.appendChild($resultTd2);
            $resultTr.appendChild($resultTd3);
            $resultTbody.appendChild($resultTr);

            var $resultDiv = rdoc.createElement('div');
            $resultDiv.appendChild($resultCaption);
            $resultDiv.appendChild($resultSummary);
            $resultDiv.appendChild($resultTable);

            data.structure = $resultDiv;

            return [
              data.hidden,
              data.structure
            ];
          },
          function () {
            var childNodes = this.childNodes,
              $caption = null,
              i,
              l;
            for (i = 0; i < childNodes.length; i++) {
              if (childNodes[i].tagName &&
                childNodes[i].tagName.toLowerCase() === 'caption') {
                $caption = childNodes[i];
                break;
              }
            }
            // var hasCaption = !!$caption;
            // 02-11 hidden, none 상태일 시도 검사
            var hasCaption = !!$caption ? (window.getComputedStyle($caption).visibility === 'hidden' ||
                window.getComputedStyle($caption).display === 'none' ? false : true) :
              false;
            var hasSummary = !!this.getAttribute('summary');
            var $theadTh = getElsFromChildNodes(
              getElsFromChildNodes(
                getElsFromChildNodes(this, 'thead'),
                'tr'
              ),
              'th'
            );
            var $tfootTh = getElsFromChildNodes(
              getElsFromChildNodes(
                getElsFromChildNodes(this, 'tfoot'),
                'tr'
              ),
              'th'
            );
            var $tbodyTh = getElsFromChildNodes(
              getElsFromChildNodes(
                getElsFromChildNodes(this, 'tbody'),
                'tr'
              ),
              'th'
            ).concat(
              getElsFromChildNodes(
                getElsFromChildNodes(this, 'tr'),
                'th'
              )
            );
            var hasTh = $theadTh.length || $tfootTh.length || $tbodyTh.length;

            var doctype = document.doctype;
            var publicId, systemId
            if (!!doctype) {
              publicId = doctype.publicId;
              systemId = doctype.systemId;
            } else {
              doctype = 'none'
            }

            if (doctype !== 'none' && !publicId && !systemId) {
              if (hasTh && hasCaption && !hasSummary) {
                return 'pass';
              } else if (!hasTh && !hasCaption && !hasSummary) {
                return 'warning';
              } else if (!hasTh && hasCaption || hasSummary) {
                return 'fail';
              } else {
                return 'fail';
              }
            } else {
              if (hasTh && hasCaption && hasSummary) {
                return 'pass';
              } else if (!hasTh && !hasCaption && !hasSummary) {
                return 'warning';
              } else if (!hasTh && hasCaption || hasSummary) {
                return 'fail';
              } else {
                return 'fail';
              }
            }
          }
        ),

        label: new TableSection(
          cwin,
          rdoc,
          '21. ' + g.achecker.i18n.get('No21'),
          'input,textarea,select',
          [{
              label: g.achecker.i18n.get('Hidden'),
              width: 45
            },
            {
              label: g.achecker.i18n.get('Element'),
              width: 55
            },
            {
              label: g.achecker.i18n.get('FormType'),
              width: 66
            },
            {
              label: g.achecker.i18n.get('LabelConnection'),
              className: 'lt',
              width: 100
            },
            {
              label: g.achecker.i18n.get('TitleAttribute'),
              width: 150
            },
            {
              label: g.achecker.i18n.get('FullHTML'),
              className: 'elementhtml',
              width: 180
            },
          ],
          isIncludeFrame,
          frameDocs,
          g.achecker.i18n.get('NotApplicable'),
          function (doc, url) {
            var typeAttr = this.getAttribute('type') ? this.getAttribute('type').toLowerCase() : null;
            if (this.tagName === 'INPUT' &&
              (typeAttr === 'submit' ||
                typeAttr === 'button' ||
                typeAttr === 'image' ||
                typeAttr === 'hidden' ||
                typeAttr === 'reset')) {
              return false;
            }

            var data = {
              hidden: '',
              el: '',
              type: '',
              label: '',
              title: '',
              html: ''
            };
            var _id = this.getAttribute('id');
            var $labels = doc.getElementsByTagName("label");
            var hasLabelElement = false,
              $label = null,
              i,
              l;

            if (_id) {
              for (i = 0, l = $labels.length; i < l; i++) {
                if ($labels[i].getAttribute('for') === _id) {
                  hasLabelElement = true;
                  $label = $labels[i];
                  break;
                }
              }
            }
            var hasTitle = !!this.getAttribute('title');

            var hasImplicitLabel = false;
            var parentEl = this.parentNode;
            do {
              if (parentEl.tagName === 'LABEL') {
                hasImplicitLabel = true;
                $label = parentEl;
                break;
              }
              parentEl = parentEl.parentNode;
            } while (parentEl.parentNode);

            data.el = this.tagName.toLowerCase();
            data.type = typeAttr || '-';
            data.label = $label ? getLabelText($label) : '';
            if (!data.label) {
              data.label = 'X';
            }
            data.title = this.getAttribute('title');
            if (!data.title) {
              data.title = '-';
            }

            data.html = this.outerHTML;

            return [
              data.hidden,
              data.el,
              data.type,
              data.label,
              data.title,
              data.html
            ];
          },
          function (doc, url) {
            var _id = this.getAttribute('id');
            var $labels = doc.getElementsByTagName("label");
            var hasLabelElement = false,
              i, l;
            var linkedLabel;
            if (_id) {
              for (i = 0, l = $labels.length; i < l; i++) {
                if ($labels[i].getAttribute('for') === _id) {
                  hasLabelElement = true;
                  linkedLabel = $labels[i];
                  break;
                }
              }
            }
            var hasTitle = !!this.getAttribute('title');
            var hasImplicitLabel = false;
            var parentEl = this.parentNode;
            //label 태그안에 img 태그가 있다면 pass로 판정(img alt값이 대신 읽어주는 사유로..)
            let hasImgTag = () => {
              for (let i = 0; i < linkedLabel.childNodes.length; i++) {
                if (linkedLabel.childNodes[i].tagName === 'IMG') {
                  return true;
                }
              }
              return false;
            }
            do {
              // label 태그가 존재 && 안의 텍스트 존재
              if (parentEl.tagName === 'LABEL' && !!parentEl.innerText) {
                hasImplicitLabel = true;
              }
              parentEl = parentEl.parentNode;
            } while (parentEl.parentNode);

            if (hasLabelElement && (getLabelText(linkedLabel) || hasImgTag())) {
              return 'pass';
            }
            if (hasImplicitLabel) {
              return 'pass';
            }
            if (hasTitle) {
              return 'warning';
            }
            return 'fail';
          }
        ),

        validation: new ToolSection(
          cwin,
          rdoc,
          'w3c_validation',
          '23. ' + g.achecker.i18n.get('No23'),
          function (win, rdoc) {
            var isChromeAddon = typeof chrome === "object" && chrome.extension;
            var isFirefoxAddon = typeof Components === "object" && Components.classes;
            if (!isChromeAddon && !isFirefoxAddon) {
              var $res = rdoc.createElement('p');
              $res.className = 'comment';
              $res.innerText = 'Not Supported.';
              $res.textContent = 'Not Supported.';
              return $res;
            }

            var getResultDetailEl = function (resultList, url, errcnt, warningcnt) {
              const $res = rdoc.createElement('div');
              $res.className = 'validationResult';
              const $errhead = rdoc.createElement('h3');
              $errhead.innerText = 'Error : ' + errcnt;
              $errhead.textContent = 'Error : ' + errcnt;
              $errhead.className = 'fail';
              const $errul = rdoc.createElement('ul');
              $errul.className = 'kwaxhide';
              $errhead.onclick = () => {
                $errul.classList.toggle("kwaxhide");
              }

              const $warninghead = rdoc.createElement('h3');
              $warninghead.innerText = 'Warning : ' + warningcnt;
              $warninghead.textContent = 'Warning : ' + warningcnt;
              $warninghead.className = 'warning';
              const $warningul = rdoc.createElement('ul');
              $warningul.className = 'kwaxhide';
              $warninghead.onclick = () => {
                $warningul.classList.toggle("kwaxhide");
              }

              //12-27 correct detail
              const $errSection = {};
              const $warningSection = {};
              for (let i = 0; i < resultList.length; i++) {
                let $li = resultList[i];
                $li.removeChild($li.children[1]);
                $li.className = $li.className + ' kwaxhide';

                let splitText = '';
                for (let i = 0; i < $li.querySelector('span').childNodes.length; i++) {
                  if ($li.querySelector('span').childNodes[i].nodeType === Node.TEXT_NODE) {
                    splitText += $li.querySelector('span').childNodes[i].textContent;
                  } else {
                    splitText += '____';
                  }
                }
                let sectionTitleKey = splitText;
                if ($li.className.includes('err')) {
                  if ($errSection[sectionTitleKey] === undefined) {
                    $errSection[sectionTitleKey] = rdoc.createElement('li');
                    let $title = rdoc.createElement('h3');

                    $title.innerText = splitText;
                    $title.onclick = () => {
                      let sectionList = $errSection[sectionTitleKey].children;
                      for (let i = 1; i < sectionList.length; i++) {
                        sectionList[i].classList.toggle('kwaxhide');
                      }
                    }
                    $errSection[sectionTitleKey].appendChild($title);
                    $errSection[sectionTitleKey].appendChild($li);
                  } else {
                    $errSection[sectionTitleKey].appendChild($li);
                  }
                } else {
                  if ($warningSection[sectionTitleKey] === undefined) {
                    $warningSection[sectionTitleKey] = rdoc.createElement('li');
                    let $title = rdoc.createElement('h3');
                    let splitText = '';
                    for (let i = 0; i < $li.querySelector('span').childNodes.length; i++) {
                      if ($li.querySelector('span').childNodes[i].nodeType === Node.TEXT_NODE) {
                        splitText += $li.querySelector('span').childNodes[i].textContent;
                      } else {
                        splitText += '____';
                      }
                    }
                    $title.innerText = splitText;
                    $title.onclick = () => {
                      let sectionList = $warningSection[sectionTitleKey].children;
                      for (let i = 1; i < sectionList.length; i++) {
                        sectionList[i].classList.toggle('kwaxhide');
                      }
                    }
                    $warningSection[sectionTitleKey].appendChild($title);
                    $warningSection[sectionTitleKey].appendChild($li);
                  } else {
                    $warningSection[sectionTitleKey].appendChild($li);
                  }
                }
              }
              for (let key in $errSection) {
                $errul.appendChild($errSection[key]);
              }
              for (let key in $warningSection) {
                $warningul.appendChild($warningSection[key]);
              }
              $res.appendChild($errhead);
              $res.appendChild($errul);
              $res.appendChild($warninghead);
              $res.appendChild($warningul);
              return $res;
            };

            var getHtml4ResultDetailEl = function (resultList, url, errcnt, warningcnt) {
              const $res = rdoc.createElement('div');
              $res.className = 'validationResult';
              const $errhead = rdoc.createElement('h3');
              $errhead.innerText = 'Error : ' + errcnt;
              $errhead.textContent = 'Error : ' + errcnt;
              $errhead.className = 'fail';
              const $errul = rdoc.createElement('ul');
              $errul.className = 'kwaxhide';
              $errhead.onclick = () => {
                $errul.classList.toggle('kwaxhide');
              }

              const $warninghead = rdoc.createElement('h3');
              $warninghead.innerText = 'Warning : ' + warningcnt;
              $warninghead.textContent = 'Warning : ' + warningcnt;
              $warninghead.className = 'warning';
              const $warningul = rdoc.createElement('ul');
              $warningul.className = 'kwaxhide';
              $warninghead.onclick = () => {
                $warningul.classList.toggle('kwaxhide');
              }

              //12-27 correct detail
              const $errSection = {};
              const $warningSection = {};
              for (let i = 0; i < resultList.length; i++) {
                let $li = resultList[i];
                $li.children[0].removeChild($li.children[0].children[0]);
                $li.className = $li.className + ' kwaxhide';
                let splitText = '';
                let textArray = $li.querySelector('.msg').textContent.split(' ');
                for (let i = 0; i < textArray.length; i++) {
                  if (!textArray[i].includes('"')) {
                    splitText += textArray[i] + " ";
                  } else {
                    splitText += '____' + " ";
                  }
                }
                let sectionTitleKey = splitText;
                if ($li.className.includes('err')) {
                  $li.removeChild($li.children[4]);
                  if ($errSection[sectionTitleKey] === undefined) {
                    $errSection[sectionTitleKey] = rdoc.createElement('li');
                    let $title = rdoc.createElement('h3');
                    $title.innerText = splitText;
                    $title.onclick = () => {
                      let sectionList = $errSection[sectionTitleKey].children;
                      for (let i = 1; i < sectionList.length; i++) {
                        sectionList[i].classList.toggle('kwaxhide');
                      }
                    }
                    $errSection[sectionTitleKey].appendChild($title);
                    $errSection[sectionTitleKey].appendChild($li);
                  } else {
                    $errSection[sectionTitleKey].appendChild($li);
                  }
                } else {
                  if ($warningSection[sectionTitleKey] === undefined) {
                    $warningSection[sectionTitleKey] = rdoc.createElement('li');
                    let $title = rdoc.createElement('h3');
                    $title.innerText = $li.querySelector('.msg').textContent;
                    $title.onclick = () => {
                      let sectionList = $warningSection[sectionTitleKey].children;
                      for (let i = 1; i < sectionList.length; i++) {
                        sectionList[i].classList.toggle('kwaxhide');
                      }
                    }
                    $warningSection[sectionTitleKey].appendChild($title);
                    $warningSection[sectionTitleKey].appendChild($li);
                  } else {
                    $warningSection[sectionTitleKey].appendChild($li);
                  }
                }
              }
              for (let key in $errSection) {
                $errul.appendChild($errSection[key]);
              }
              for (let key in $warningSection) {
                $warningul.appendChild($warningSection[key]);
              }

              $res.appendChild($errhead);
              $res.appendChild($errul);
              $res.appendChild($warninghead);
              $res.appendChild($warningul);
              return $res;
            };

            var getItemEl = function (url) {
              var el = rdoc.getElementById("w3c_validation");
              var itemEls = rdoc.querySelectorAll("#w3c_validation li.validationItem");
              var i;

              for (i = 0; i < itemEls.length; i++) {
                var urlEl = itemEls[i].getElementsByClassName("url")[0];
                if (urlEl.innerText === url || urlEl.textContent === url) {
                  return itemEls[i];
                }
              }
            };

            var runValidator = function (url, doc) {
              var req2 = new XMLHttpRequest();
              var ggTimeout = setTimeout(function () {
                var itemEl = getItemEl(url);
                var errcntEl = itemEl.getElementsByClassName("errcnt")[0];
                var directValidationLink = rdoc.createElement("a");
                directValidationLink.target = '_blank';
                directValidationLink.href = 'http://validator.w3.org/check?uri=' + encodeURIComponent(url);
                directValidationLink.textContent = '(' + g.achecker.i18n.get('ValidateManually') + ')';
                errcntEl.innerText = g.achecker.i18n.get('ValidationTimeout') + ' ';
                errcntEl.textContent = g.achecker.i18n.get('ValidationTimeout') + ' ';
                errcntEl.appendChild(directValidationLink);
              }, 60 * 1000);

              req2.onreadystatechange = function () {
                try {
                  if (req2.readyState === XMLHttpRequest.DONE && req2.status === 200) {
                    if (req2.responseURL !== "https://validator.w3.org/check") {
                      clearTimeout(ggTimeout);
                      var responseText = req2.responseText;
                      responseText = responseText.substring(responseText.indexOf('<ol>'), responseText.indexOf('</ol>') + 5);
                      var temp = document.createElement('ol');
                      temp.innerHTML = responseText;
                      var resultList = temp.querySelectorAll('.error, .warning');
                      var headerEl = rdoc.querySelectorAll("#w3c_validation h2")[0];
                      var itemEl = getItemEl(url);
                      var errcnt = 0;
                      var warningcnt = 0;
                      for (let i = 0; i < resultList.length; i++) {
                        if (resultList[i].className === 'error') {
                          errcnt++;
                        } else if (resultList[i].className === 'info warning') {
                          warningcnt++;
                        }
                      }

                      var errcntEl = itemEl.getElementsByClassName("errcnt")[0];
                      errcntEl.innerText = errcnt + ' Errors  ' + warningcnt + '  Warnings';
                      errcntEl.textContent = errcnt + ' Errors  ' + warningcnt + '  Warnings';
                      itemEl.className = errcnt > 0 ? 'fail' : warningcnt > 0 ? 'warning' : 'pass';
                      if (errcnt > 0) {
                        headerEl.className += " fail";
                      } else if (warningcnt > 0) {
                        headerEl.className += " warning";
                      }
                      var $res = getResultDetailEl(resultList, url, errcnt, warningcnt);
                      itemEl.appendChild($res);
                    } else {
                      clearTimeout(ggTimeout);
                      var responseText = req2.responseText;
                      responseText = responseText.substring(responseText.indexOf('<ol id="error_loop">'), responseText.lastIndexOf('</ol>') + 5);
                      var temp = document.createElement('ol');
                      temp.innerHTML = responseText;
                      var resultList = temp.querySelectorAll('.msg_err, .msg_info');
                      var headerEl = rdoc.querySelectorAll("#w3c_validation h2")[0];
                      var itemEl = getItemEl(url);
                      var errcnt = 0;
                      var warningcnt = 0;
                      for (let i = 0; i < resultList.length; i++) {
                        if (resultList[i].className === 'msg_err') {
                          errcnt++;
                        } else if (resultList[i].className === 'msg_info') {
                          warningcnt++;
                        }
                      }

                      var errcntEl = itemEl.getElementsByClassName("errcnt")[0];
                      errcntEl.innerText = errcnt + ' Errors  ' + warningcnt + '  Warnings';
                      errcntEl.textContent = errcnt + ' Errors  ' + warningcnt + '  Warnings';
                      itemEl.className = errcnt > 0 ? 'fail' : warningcnt > 0 ? 'warning' : 'pass';
                      if (errcnt > 0) {
                        headerEl.className += " fail";
                      } else if (warningcnt > 0) {
                        headerEl.className += " warning";
                      }
                      var $res = getHtml4ResultDetailEl(resultList, url, errcnt, warningcnt);
                      itemEl.appendChild($res);
                    }
                  }
                } catch (e) {
                  console.log(e);
                }
              };

              try {
                let doctype = new XMLSerializer().serializeToString(doc.doctype);

                var data = new FormData();
                data.append('showsource', 'no');
                data.append('fragment', doctype + doc.documentElement.outerHTML);
                req2.open("POST", "https://validator.w3.org/check", true);
                req2.send(data);
              } catch (e) {}
            };

            var doValidation = function (url, doc) {
              runValidator(url, doc);
            };

            var urls = [],
              docs = [],
              i, l;
            if (win.location.href.substr(0, 7) === 'http://' ||
              win.location.href.substr(0, 8) === 'https://' ||
              win.location.href.substr(0, 7) === 'file://') {
              urls.push(win.location.href);
              docs.push(win.document);
            }
            // ignore frame docs
            // if (isIncludeFrame) {
            //   for (i = 0, l = frameDocs.length; i < l; i++) {
            //     var _url = frameDocs[i].src;
            //     if (_url.substr(0, 7) === 'http://' || _url.substr(0, 8) === 'https://') {
            //       urls.push(_url);
            //       docs.push(frameDocs[i]);
            //     }
            //   }
            // }

            var $output = rdoc.createElement('ul');
            for (i = 0, l = urls.length; i < l; i++) {
              doValidation(urls[i], docs[i]);
              var $item = rdoc.createElement('li');
              $item.className = 'validationItem';
              var $_url = rdoc.createElement('span');
              $_url.className = 'url';
              $_url.innerText = urls[i];
              $_url.textContent = urls[i];
              var $_errcnt = rdoc.createElement('b');
              $_errcnt.className = 'errcnt';
              $_errcnt.innerText = g.achecker.i18n.get('Loading');
              $_errcnt.textContent = g.achecker.i18n.get('Loading');
              $item.appendChild($_url);
              $item.appendChild(rdoc.createTextNode(': '));
              $item.appendChild($_errcnt);
              $output.appendChild($item);
            }
            return $output;
          }
        )
      },

      buttonBar: (function () {

        let buttonBar = rdoc.createElement('div');
        let div = rdoc.createElement('div');
        div.className = 'btn-ui-list';
        buttonBar.className = 'btn-ui';

        //let $foldBtn = rdoc.createElement('img');
        let $disableCssBtn = KWAX.button.disableCssBtn(rdoc);
        // 2019.10.16작업중
        let $speakBtn = KWAX.button.speakBtn(rdoc);

//        let $disableCssBtn = KWAX.button.disableCssBtn(rdoc);
/*
        let $etcFuncBtn = KWAX.button.etcFuncBtn(rdoc);
        let $saveBtn = KWAX.button.saveBtn(rdoc);
        let $captureBtn = KWAX.button.captureBtn(rdoc);
        let $tabFocusBtn = KWAX.button.tabFocusBtn(rdoc);
        let $changeLocationBtn = KWAX.button.changeLocationBtn(rdoc);
        let $downloadStatisticsXLSXBtn = KWAX.button.downloadStatisticsXLSX(rdoc);
*/        
        // let $downloadStatisticsBinaryBtn = KWAX.button.downloadStatisticsBinaryBtn(rdoc);

        //fold 버튼 설정
        // $foldBtn.className = 'fold';
        // $foldBtn.setAttribute('data-folded', 'folded');
        // $foldBtn.setAttribute('src', ""+chrome.extension.getURL('/images/toggle-off-32.png')+"");
        // $foldBtn.title = g.achecker.i18n.get('UnfoldAll');
        // $foldBtn.alt = g.achecker.i18n.get("UnfoldAll");
        // $foldBtn.textContent = 'Toggle All';
        // $foldBtn.onclick = function () {
        //   var $headings = rdoc.querySelectorAll('.waxSection h2,.waxFrames h2');
        //   var foldedClass = this.getAttribute('data-folded') === 'folded' ? '' : 'folded';
        //   var i, l;

        //   for (i = 0, l = $headings.length; i < l; i++) {
        //     $headings[i].className = $headings[i].className.replace('folded', '') + ' ' + foldedClass;
        //   }

        //   if (foldedClass) {
        //     this.className = 'unfold';
        //     this.title = g.achecker.i18n.get('UnfoldAll');
        //     this.alt = g.achecker.i18n.get("UnfoldAll");
        //     this.setAttribute('src', ""+chrome.extension.getURL('/images/toggle-off-32.png')+"");
        //   } else {
        //     this.className = 'fold';
        //     this.title = g.achecker.i18n.get('FoldAll');
        //     this.alt = g.achecker.i18n.get("FoldAll");
        //     this.setAttribute('src', ""+chrome.extension.getURL('/images/toggle-on-32.png')+"");
        //   }
        //   this.setAttribute('data-folded', foldedClass);
        //   // fix bug: IE8 won't reflow when set data-* attribute
        //   rdoc.body.className = rdoc.body.getAttribute("className");
        // };

        // popup element
        let $popup = rdoc.createElement('div');
        $popup.className = 'popup tagname';
        $popup.id = 'attachPopUp';
        fetch(chrome.extension.getURL('/popup-attach-tagname.html'))
          .then(response => response.text())
          .then(data => {
            $popup.innerHTML += data;
          }).catch(err => {
            // handle error
            console.log(err);
          });


        div.appendChild($popup);
        div.appendChild($speakBtn);
/*
        div.appendChild($etcFuncBtn);

        div.appendChild($changeLocationBtn);
        div.appendChild($disableCssBtn);
        div.appendChild($tabFocusBtn);

        div.appendChild($captureBtn);
        div.appendChild($saveBtn);
        div.appendChild($downloadStatisticsXLSXBtn);
*/        
        // div.appendChild($downloadStatisticsBinaryBtn);


        buttonBar.appendChild(div);
        return buttonBar;
      }()),

      // filterBar: (function () {

      //   let filterBar = rdoc.createElement('div');
      //   let $filterBtn = KWAX.button.filterBtn(rdoc);
      //   filterBar.className = 'tab-ui';

      //   filterBar.appendChild($filterBtn);

      //   return filterBar;

      // }()),


      // foldBar: (function () {

      //   let foldBar = rdoc.createElement('div');
      //   foldBar.id = "foldbar-ui"
      //   let $foldBtn = KWAX.button.foldBtn(rdoc);
      //   //   foldBar.className = 'foldbar-ui';

      //   foldBar.appendChild($foldBtn);

      //   return foldBar;

      // }()), 
      
      speakBtn: (function () {

        let speakBtn = rdoc.createElement('div');
        speakBtn.id = "speak-ui"
        let $speakBtn = KWAX.button.speakBtn(rdoc);
        //   foldBar.className = 'foldbar-ui';
        speakBtn.appendChild($speakBtn);
        return speakBtn;
      }())

    };
  };
}(this, this.document));